#!/usr/bin/env python


"""

test_nmrDataSet

author(s): C. Ludwig
origin: 26-07-2019


"""


import unittest
import metabolabpy.nmr.nmrDataSet as nmrDataSet
import os
import numpy as np
import shutil

class nmrDataSetTestCase(unittest.TestCase):
    
    def test_readSpc(self):
        pName = os.path.join(os.path.dirname(__file__),"data","nmrData") # directory of test data set
        eName = "1"                                                      # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()                                 # create nmrDataSet object
        nd.readSpc(pName,eName)                                      # check if Bruker data can be read
        self.assertEqual(len(nd.nmrdat[0][0].fid[0]), 32768)         # check number of data points in fid
        self.assertEqual(len(nd.nmrdat[0][0].spc[0]), 65536)         # check number of data points in 1r

    def test_ft(self):
        pName = os.path.join(os.path.dirname(__file__),"data","nmrData") # directory of test data set
        eName = "1"                                                      # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()                                 # create nmrDataSet object
        nd.readSpc(pName,eName)                                      # check if Bruker data can be read
        nd.ft()
        self.assertEqual(len(nd.nmrdat[0][0].spc[0]), 65536)         # check number of data points in Fourier transformed spectrum

    def test_ftAll(self):
        pName = os.path.join(os.path.dirname(__file__),"data","nmrData") # directory of test data set
        eName = "1"                                                      # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()                                 # create nmrDataSet object
        nd.readSpc(pName,eName)                                      # check if Bruker data can be read
        nd.ftAll()
        self.assertEqual(len(nd.nmrdat[0][0].spc[0]), 65536)         # check number of data points in Fourier transformed spectrum

    def test_load(self):
        nd = nmrDataSet.NmrDataSet()
        dataSetPath = os.path.join(os.path.dirname(__file__), "data", "loadData.mlpy")
        nd.load(dataSetPath)
        self.assertEqual(nd.nmrdat[0][0].refShift[0], 0.0)
        self.assertEqual(nd.nmrdat[0][0].refPoint[0], 10342)

    def test_setZeroFill(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.setZeroFill([131072])
        nd.ft()
        self.assertEqual(len(nd.nmrdat[0][0].spc[0]),131072)  # check number of data points in Fourier transformed spectrum

    def test_autobaseline1d(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.autobaseline1d()

    def test_autobaseline1dAll(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.autobaseline1dAll()

    def test_autophase1d(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.autophase1d()

    def test_autophase1dAll(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.autophase1dAll()

    def test_autoref(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.autoref()
        self.assertEqual(nd.nmrdat[0][0].refPoint[0], 10342)
        self.assertEqual(nd.nmrdat[0][0].refShift[0], 0.0)
        nd = [[]]
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "3"  # 2D HSQC NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.ft()
        nd.autoref()
        self.assertEqual(nd.nmrdat[0][0].refPoint[0], 152)
        self.assertEqual(nd.nmrdat[0][0].refShift[0], 0.0)
        self.assertEqual(nd.nmrdat[0][0].refPoint[1], 1792)
        self.assertAlmostEqual(nd.nmrdat[0][0].refShift[1], 80.0, 4)

    def test_clear(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.clear()
        self.assertEqual(len(nd.nmrdat),1)
        self.assertEqual(len(nd.nmrdat[0]),0)

    def test_excludeRegion(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.pp.excludeStart = [8.9]
        nd.pp.excludeEnd   = [11.0]
        nd.pp.flagExcludeRegion = True
        nd.dataPreProcessing()
        pts = len(nd.nmrdat[0][0].spc[0]) - nd.nmrdat[0][0].ppm2points([nd.pp.excludeStart, nd.pp.excludeEnd],0)
        pts2 = range(int(min(pts)), int(max(pts)))
        ssum = np.sum(nd.nmrdat[0][0].spc[0][pts2].real)
        self.assertEqual(ssum,0.0)
        nd = [[]]
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "2"  # 2D Jres NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.ft()
        nd.autoref()
        nd.pjres(2, 'skyline')
        nd.pp.excludeStart = [8.9]
        nd.pp.excludeEnd   = [10.5]
        nd.pp.flagExcludeRegion = True
        nd.s = 1
        nd.e = 0
        nd.dataPreProcessing()
        pts = len(nd.nmrdat[1][0].spc[0]) - nd.nmrdat[1][0].ppm2points([nd.pp.excludeStart, nd.pp.excludeEnd],0)
        pts2 = range(int(min(pts)), int(max(pts)))
        ssum = np.sum(nd.nmrdat[1][0].spc[0][pts2].real)
        self.assertEqual(ssum,0.0)

    def test_exportDataSet(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.pp.excludeStart = [8.9]
        nd.pp.excludeEnd   = [11.0]
        nd.pp.flagExcludeRegion = True
        nd.pp.bucketPoints = 64
        nd.pp.flagBucketSpectra = True
        nd.dataPreProcessing()
        nd.pp.exportExcelPath = os.path.expanduser("~")
        nd.pp.exportPathName = os.path.expanduser("~")
        nd.pp.exportMetaboAnalystPath = os.path.expanduser("~")
        nd.pp.exportrDolphinPath = os.path.expanduser("~")
        nd.pp.exportBatmanPath = os.path.expanduser("~")
        nd.pp.exportBrukerPath = os.path.expanduser("~")
        nd.pp.exportFileName = "testExport.csv"
        nd.pp.exportSamplesInRowsCols = 0
        nd.pp.classSelect = ["1"]
        nd.pp.exportMethod = 1
        nd.exportDataSet()
        self.assertEqual(os.path.isfile(nd.pp.exportPathName + os.sep + nd.pp.exportFileName), True)
        os.remove(nd.pp.exportPathName + os.sep + nd.pp.exportFileName)
        nd.pp.exportFileName = "testExport.csv"
        nd.pp.exportSamplesInRowsCols = 1
        nd.pp.exportMethod = 1
        nd.exportDataSet()
        self.assertEqual(os.path.isfile(nd.pp.exportPathName + os.sep + nd.pp.exportFileName), True)
        os.remove(nd.pp.exportPathName + os.sep + nd.pp.exportFileName)
        nd.pp.exportExcel = "testExport.xlsx"
        nd.pp.exportSamplesInRowsCols = 0
        nd.pp.exportMethod = 0
        nd.exportDataSet()
        self.assertEqual(os.path.isfile(nd.pp.exportExcelPath + os.sep + nd.pp.exportExcel), True)
        os.remove(nd.pp.exportExcelPath + os.sep + nd.pp.exportExcel)
        nd.pp.exportExcel = "testExport.xlsx"
        nd.pp.exportSamplesInRowsCols = 1
        nd.pp.exportMethod = 0
        nd.exportDataSet()
        self.assertEqual(os.path.isfile(nd.pp.exportExcelPath + os.sep + nd.pp.exportExcel), True)
        os.remove(nd.pp.exportExcelPath + os.sep + nd.pp.exportExcel)
        nd.pp.exportMetaboAnalyst = "metaboAnalystExport.csv"
        nd.pp.exportSamplesInRowsCols = 0
        nd.pp.exportMethod = 2
        nd.exportDataSet()
        self.assertEqual(os.path.isfile(nd.pp.exportMetaboAnalystPath + os.sep + nd.pp.exportMetaboAnalyst), True)
        os.remove(nd.pp.exportMetaboAnalystPath + os.sep + nd.pp.exportMetaboAnalyst)
        nd.pp.exportrDolphin = "rDolphinExport.csv"
        nd.pp.exportSamplesInRowsCols = 0
        nd.pp.exportMethod = 3
        nd.exportDataSet()
        self.assertEqual(os.path.isfile(nd.pp.exportrDolphinPath + os.sep + nd.pp.exportrDolphin), True)
        os.remove(nd.pp.exportrDolphinPath + os.sep + nd.pp.exportrDolphin)
        nd.pp.exportBatman = "batmanExport.txt"
        nd.pp.exportSamplesInRowsCols = 0
        nd.pp.exportMethod = 4
        nd.exportDataSet()
        self.assertEqual(os.path.isfile(nd.pp.exportBatmanPath + os.sep + nd.pp.exportBatman), True)
        os.remove(nd.pp.exportBatmanPath + os.sep + nd.pp.exportBatman)
        nd.pp.exportBruker = "brukerExport"
        nd.pp.exportSamplesInRowsCols = 0
        nd.pp.exportMethod = 5
        nd.exportDataSet()
        self.assertEqual(os.path.isfile(nd.pp.exportBrukerPath + os.sep + nd.pp.exportBruker + os.sep + '1' + os.sep + 'acqus'), True)
        shutil.rmtree(nd.pp.exportBrukerPath + os.sep + nd.pp.exportBruker)

    def test_noiseFiltering(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.pp.noiseStart     = 10.0
        nd.pp.noiseEnd       = 10.5
        nd.pp.noiseThreshold = 8.0
        nd.pp.thLineWidth    = 2.0
        nd.pp.flagNoiseFiltering = True
        nd.dataPreProcessing()
        pts = len(nd.nmrdat[0][0].spc[0]) - nd.nmrdat[0][0].ppm2points([9.5, 11.5],0)
        pts2 = range(int(min(pts)), int(max(pts)))
        ssum = np.sum(nd.nmrdat[0][0].spc[0][pts2].real)
        self.assertEqual(ssum,0.0)

    def test_bucketSpectra(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.pp.bucketPoints = 64
        nd.pp.flagBucketSpectra = True
        nd.dataPreProcessing()
        pts = len(nd.nmrdat[0][0].spc[0])
        self.assertEqual(pts,1024)

    def test_compressBuckets(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.nmrdat[0][0].proc.ph0[0] = -18.49143139318872
        nd.nmrdat[0][0].proc.ph1[0] = -0.7082407011136772
        nd.ft()
        nd.pp.compressStart = [6.5448]
        nd.pp.compressEnd = [7.4705]
        nd.pp.flagCompressBuckets = True
        nd.dataPreProcessing()
        pts = nd.nmrdat[0][0].ppm2points([nd.pp.compressEnd[0], nd.pp.compressStart[0]], 0)
        npts = len(nd.nmrdat[0][0].spc[0])
        spc = nd.nmrdat[0][0].spc[0][npts - pts[0]:npts - pts[1]].real
        self.assertEqual(spc.max(), spc.sum())

    def test_avoidNegativeValues(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.nmrdat[0][0].proc.ph0[0] = -18.49143139318872
        nd.nmrdat[0][0].proc.ph1[0] = -0.7082407011136772
        nd.ft()
        nd.pp.avoidNegativeValues = True
        nd.dataPreProcessing()
        self.assertEqual(nd.nmrdat[0][0].spc[0].real.min(), 0.0)

    def test_segmentalAlignment(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "10"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.nmrdat[0][0].proc.ph0[0] = -85.94556
        nd.nmrdat[0][0].proc.ph1[0] =   6.64201
        nd.nmrdat[0][0].proc.lb[0] = 2.0
        nd.ft()
        nd.nmrdat[0][0].autobaseline1d()
        nd.nmrdat[0][0].autoRef()
        eName = "11"  # 1D NMR data in exp 1
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.nmrdat[0][1].proc.ph0[0] = -79.29513383337031
        nd.nmrdat[0][1].proc.ph1[0] =   7.790498486770286
        nd.nmrdat[0][1].proc.lb[0] = 2.0
        nd.e = 1
        nd.ft()
        nd.nmrdat[0][0].autobaseline1d()
        nd.nmrdat[0][1].autobaseline1d()
        nd.nmrdat[0][0].autoRef()
        nd.nmrdat[0][1].autoRef()
        startPpm = 8.1091
        endPpm = 8.1825
        nd.pp.segStart = [startPpm]
        nd.pp.segEnd = [endPpm]
        nd.pp.flagSegmentalAlignment = True
        nd.pp.segAlignRefSpc = 1
        nd.dataPreProcessing()
        pts1 = len(nd.nmrdat[0][0].spc[0]) - nd.nmrdat[0][0].ppm2points([endPpm, startPpm], 0)
        spc1 = nd.nmrdat[0][0].spc[0][pts1[0]:pts1[1]].real
        spc2 = nd.nmrdat[0][1].spc[0][pts1[0]:pts1[1]].real
        self.assertEqual(np.argmax(spc1), np.argmax(spc2))

    def test_scaleSpectra(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "10"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.nmrdat[0][0].proc.ph0[0] = -85.94556
        nd.nmrdat[0][0].proc.ph1[0] =   6.64201
        nd.nmrdat[0][0].proc.lb[0] = 0.3
        nd.ft()
        nd.nmrdat[0][0].autobaseline1d()
        nd.nmrdat[0][0].autoRef()
        eName = "11"  # 1D NMR data in exp 1
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.nmrdat[0][1].proc.ph0[0] = -79.29513383337031
        nd.nmrdat[0][1].proc.ph1[0] =   7.790498486770286
        nd.nmrdat[0][1].proc.lb[0] = 0.3
        nd.e = 1
        nd.ft()
        nd.pp.flagScaleSpectra = True
        nd.pp.scaleSpectraRefSpc = 1
        self.assertAlmostEqual(nd.nmrdat[0][1].spc[0].real.max() / 10331062121.45446, 1.0, 1)
        nd.pp.scalePQN = True
        nd.dataPreProcessing()
        self.assertAlmostEqual(nd.nmrdat[0][1].spc[0].real.max() / 7644944702.898952, 1.0, 1)
        nd.resetDataPreProcessing()
        self.assertAlmostEqual(nd.nmrdat[0][0].spc[0].real.max() / 9406699558.794441, 1.0, 1)
        nd.pp.scalePQN = False
        nd.pp.flagScaleSpectra = True
        nd.pp.preserveOverallScale = True
        nd.dataPreProcessing()
        self.assertAlmostEqual(nd.nmrdat[0][0].spc[0].real.max() / 11455353501.485264, 1.0, 1)
        nd.resetDataPreProcessing()
        nd.pp.preserveOverallScale = False
        nd.dataPreProcessing()
        self.assertAlmostEqual(nd.nmrdat[0][0].spc[0].real.max() / 0.011113980938806603, 1.0, 1)



    def test_pjres(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "2"  # 2D JresNMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.ft()
        nd.pjres(2, 'skyline')
        self.assertEqual(len(nd.nmrdat[1][0].spc[0]), 8192)

    def test_preProcInit(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.pp.init(1)
        self.assertEqual(nd.pp.classSelect, "1")
        # end test_preProcInit

    def test_preProcInitPlotColours(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.pp.initPlotColours(0.4, 0.8, 0.4)
        self.assertEqual(nd.pp.plotColours[0], (0.0, 0.0, 0.4))
        self.assertEqual(nd.pp.plotColours[-1], (0.8, 0.4, 0.8))
        # end test_preProcInitPlotColours

    def test_preProcSetAlpha(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        dd = nd.pp.setAlpha(0.5)
        self.assertEqual(nd.pp.alpha, 0.5)
        # end test_preProcSetAlpha

    def test_readSpc(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        self.assertEqual(len(nd.nmrdat[0][0].spc[0]), 65536)

    def test_readSpcs(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = ["1", "1"]  # 1D data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpcs([pName], eName)  # check if Bruker data can be read
        self.assertEqual(len(nd.nmrdat[0][0].spc[0]), 65536)
        self.assertEqual(len(nd.nmrdat[0][1].spc[0]), 65536)

    def test_readNMRPipeSpc(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "5"  # 2D HSQC NMRPipe data in exp 5
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readNMRPipeSpc(pName, eName)
        self.assertEqual(len(nd.nmrdat[0][0].spc), 16384)
        self.assertEqual(len(nd.nmrdat[0][0].spc[0]), 922)

    def test_readNMRPipeSpcs(self):
        pName = [os.path.join(os.path.dirname(__file__), "data", "nmrData")]  # directory of test data set
        eName = ["5"]  # 2D HSQC NMRPipe data in exp 5
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readNMRPipeSpcs(pName, eName)
        self.assertEqual(len(nd.nmrdat[0][0].spc), 16384)
        self.assertEqual(len(nd.nmrdat[0][0].spc[0]), 922)

    def test_resetDataPreProcessing(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.pp.excludeStart = [8.9]
        nd.pp.excludeEnd   = [11.0]
        nd.pp.flagExcludeRegion = True
        nd.dataPreProcessing()
        nd.resetDataPreProcessing()
        self.assertEqual(len(np.where(nd.nmrdat[0][0].spc[0].real == 0)[0]), 0)

    def test_save(self):
        nd = nmrDataSet.NmrDataSet()
        dataSetPath = os.path.join(os.path.dirname(__file__), "data", "loadData.mlpy")
        nd.load(dataSetPath)
        dataSetPath = os.path.join(os.path.dirname(__file__), "data", "saveData.mlpy")
        nd.save(dataSetPath)
        self.assertEqual(os.path.exists(dataSetPath), True)
        shutil.rmtree(dataSetPath)

    def test_setGb(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.setGb([15.0])
        self.assertEqual(nd.nmrdat[0][0].proc.gb[0],15.0)

    def test_setLb(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.setLb([15.0])
        self.assertEqual(nd.nmrdat[0][0].proc.lb[0],15.0)

    def test_setPhFromExp(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.setPhFromExp()
        self.assertEqual(nd.nmrdat[0][0].proc.ph0[0],22.0)
        self.assertEqual(nd.nmrdat[0][0].proc.ph1[0],22.0)

    def test_ph0(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.setPh0([15.0])
        self.assertEqual(nd.nmrdat[0][0].proc.ph0[0],15.0)

    def test_ph1(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.setPh1([15.0])
        self.assertEqual(nd.nmrdat[0][0].proc.ph1[0],15.0)

    def test_setSsb(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.setSsb([1.0])
        self.assertEqual(nd.nmrdat[0][0].proc.ssb[0],1.0)

    def test_setWindowType(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.setWindowType([2])
        self.assertEqual(nd.nmrdat[0][0].proc.windowType[0],2)

    def test_shiftRef(self):
        pName = os.path.join(os.path.dirname(__file__), "data", "nmrData")  # directory of test data set
        eName = "1"  # 1D NMR data in exp 1
        nd = nmrDataSet.NmrDataSet()  # create nmrDataSet object
        nd.readSpc(pName, eName)  # check if Bruker data can be read
        nd.shiftRef()
        self.assertEqual(nd.nmrdat[0][0].refShift[0],0.0)




if __name__ == "__main__":
    unittest.main()


#!/usr/bin/env python


"""

test_nmrConfig

author(s): C. Ludwig
origin: 10-06-2020


"""


import unittest
import metabolabpy.nmr.nmrConfig as nmrConfig
import os
import configparser


class nmrConfigTestCase(unittest.TestCase):

    def test_makeConfig(self):
        cf = nmrConfig.NmrConfig()
        c = cf.makeConfig()
        self.assertEqual(c['GUI']['autoplot'], 'yes')

    def test_saveConfig(self):
        cf = nmrConfig.NmrConfig()
        pName = os.path.join(os.path.dirname(__file__), "data")  # directory of test data
        cf.homeDir = pName
        cf.configFile = os.path.join(cf.homeDir, cf.fName)
        cf.saveConfig()
        self.assertEqual(os.path.isfile(cf.configFile), True)
        os.remove(cf.configFile)

    def test_readConfig(self):
        cf = nmrConfig.NmrConfig()
        pName = os.path.join(os.path.dirname(__file__), "data")  # directory of test data
        cf.homeDir = pName
        cf.configFile = os.path.join(cf.homeDir, cf.fName)
        cf.saveConfig()
        cf.readConfig()
        os.remove(cf.configFile)
        self.assertEqual(cf.keepZoom, True)

    def test_set_autoplot(self):
        cf = nmrConfig.NmrConfig()
        cf.set_autoplot("no")
        self.assertEqual(cf.autoPlot, False)
        cf.set_autoplot("yes")
        self.assertEqual(cf.autoPlot, True)

    def test_set_keepzoom(self):
        cf = nmrConfig.NmrConfig()
        cf.set_keepzoom("no")
        self.assertEqual(cf.keepZoom, False)
        cf.set_keepzoom("yes")
        self.assertEqual(cf.keepZoom, True)

    def test_set_fontsize(self):
        cf = nmrConfig.NmrConfig()
        cf.set_fontsize("18")
        self.assertEqual(cf.fontSize, 18.0)
        cf.set_fontsize("21")
        self.assertEqual(cf.fontSize, 21.0)

    def test_set_phasereferencecolour(self):
        cf = nmrConfig.NmrConfig()
        cf.set_phasereferencecolour("Black")
        self.assertEqual(cf.phaseReferenceColour, "Black")
        cf.set_phasereferencecolour("Red")
        self.assertEqual(cf.phaseReferenceColour, "Red")

    def test_setValues(self):
        cf = nmrConfig.NmrConfig()
        cf.setValues("autoplot", "no")
        self.assertEqual(cf.autoPlot, False)
        cf.setValues("autoplot", "yes")
        self.assertEqual(cf.autoPlot, True)


if __name__ == "__main__":
    unittest.main()


'''
NMR data pre-processing

'''

import numpy as np
import os


class NmrPreProc:
    
    def __init__(self):
        self.excludeStart              = np.array([])
        self.excludeEnd                = np.array([])
        self.segStart                  = np.array([])
        self.segEnd                    = np.array([])
        self.compressStart             = np.array([])
        self.compressEnd               = np.array([])
        self.segAlignRefSpc            = 0                           # use mean spectra
        self.noiseThreshold            = 4.0                         # times std of noise region
        self.noiseStart                = 10.0
        self.noiseEnd                  = 10.5
        self.bucketPoints              = 0
        self.bucketPPM                 = 0.005
        self.compressBuckets           = False
        self.scaleSpc                  = ""
        self.varianceStabilisation     = ""
        self.autoScaling               = False
        self.paretoScaling             = True
        self.gLogTransform             = False
        self.varLambda                 = 1e-5
        self.varY0                     = 0.0
        self.pName                     = ""
        self.fName                     =""
        self.samplesInRows             = True
        self.plotSelect                = np.array([])
        self.classSelect               = np.array([])
        self.plotColours               = []
        self.preProcFill               = False
        self.alpha                     = 1.0
        self.colour                    = 'gray'
        self.thColour                  = 'red'
        self.thLineWidth               = 2.0
        self.flagExcludeRegion         = False
        self.flagSegmentalAlignment    = False
        self.flagNoiseFiltering        = False
        self.flagBucketSpectra         = False
        self.flagCompressBuckets       = False
        self.flagScaleSpectra          = False
        self.flagVarianceStabilisation = False
        self.flagExportDataSet         = False
        self.scalePQN                  = True
        self.scaleSpectraRefSpc        = 0                           # use mean spectra
        self.preserveOverallScale      = False
        self.stdVal                    = 0.0
        self.exportPathName            = os.getcwd()
        self.exportFileName            = "csvExport.csv"
        self.exportExcelPath            = os.getcwd()
        self.exportExcel               = "excelExport.xlsx"
        self.exportMetaboAnalystPath   = os.getcwd()
        self.exportMetaboAnalyst       = "metaboAnalystExport.csv"
        self.exportrDolphinPath        = os.getcwd()
        self.exportrDolphin            = "rDolphinExport.csv"
        self.exportBatmanPath          = os.getcwd()
        self.exportBatman              = "NMRdata.txt"
        self.exportBrukerPath          = os.getcwd()
        self.exportBruker              = "brukerExport"
        self.exportDelimiterTab        = True
        self.exportCharacter           = ","
        self.exportSamplesInRowsCols   = 0
        self.exportMethod              = 0
        self.avoidNegativeValues       = False
        # end __init__

    def __str__(self): # pragma: no cover
        strStr = "NMR data pre-processing"
        return strStr
        # end __str__

    def init(self, nspc):
        self.plotSelect     = np.arange(nspc)
        self.classSelect    = np.empty(nspc, dtype = 'str')
        for k in range(nspc):
            self.classSelect[k] = "1"
            
        self.initPlotColours()
        return "pre-processing initialised"
        # end init
        
    def initPlotColours(self, int1 = 0.4, int2 = 0.8, int3 = 0.4):
        self.plotColours = [( 0.0,  0.0, int1),
                            (int1,  0.0,  0.0),
                            ( 0.0, int1,  0.0),
                            ( 0.0, int1, int1),
                            (int1, int1,  0.0),
                            (int1,  0.0, int1),
                            (int3, int3, int2),
                            (int2, int3, int3),
                            (int3, int2, int3),
                            (int3, int2, int2),
                            (int2, int2, int3),
                            (int2, int3, int2)]
        # end initPlotColours
    
    def setAlpha(self, alpha):
        self.alpha = alpha
        return "Alpha set"
        # end setAlpha
        
    



import numpy as np # pragma: no cover
import os # pragma: no cover
import pickle # pragma: no cover

from metabolabpy.nmr import nmrData as nd # pragma: no cover
from metabolabpy.nmr import nmrPreProc as npp # pragma: no cover
import math # pragma: no cover
from openpyxl import Workbook # pragma: no cover
from string import ascii_uppercase # pragma: no cover
import itertools # pragma: no cover
import webbrowser # pragma: no cover
import matplotlib # pragma: no cover
import matplotlib.pyplot as pl  # pragma: no cover

class NmrDataSet:

    def __init__(self):
        self.nmrdat            = [[]]
        self.s                 = 0
        self.e                 = -1
        self.pp                = npp.NmrPreProc()
        self.deselect          = np.array([])
        self.deselect2         = np.array([])
        self.cmdBuffer         = np.array([])
        self.cmdIdx            = -1
        self.script            = ""
        self.console           = ""
        self.fileFormatVersion = 0.1
        self.keepZoom          = True
        # end __init__

    def __str__(self): # pragma: no cover
        rString  = 'MetaboLabPy NMR Data Set (v. 0.1)\n'
        rString += '__________________________________________________________________\n'
        rString += ' Number of data sets\t\t\t: {:0.0f}\n'.format(len(self.nmrdat))
        rString += ' Number of NMR spectra in current data set\t: {:0.0f}\n'.format(len(self.nmrdat[self.s]))
        if(len(self.nmrdat[self.s])>0):
            rString += ' Current data set/exp\t\t\t: {:0.0f}/{:0.0f}\n'.format(self.s + 1,self.e + 1)
            rString += '__________________________________________________________________\n'
            rString += 'Current title file: \n'
            rString += self.nmrdat[self.s][self.e].title
        return rString
        # end __str__

    def autobaseline1d(self):
        if(len(self.nmrdat)>0):
            if(len(self.nmrdat[self.s])>0):
                self.nmrdat[self.s][self.e].autobaseline1d()
                
                
        # end autobaseline1d

    def autobaseline1dAll(self):
        nExp    = len(self.nmrdat[self.s])
        origExp = self.e
        for k in range(nExp):
            self.e = k
            self.autobaseline1d()
    
        self.e = origExp
        return "Finished autobaseline1dAll"
        # end autobaseline1dAll

    def autophase1d(self):
        if(len(self.nmrdat)>0):
            if(len(self.nmrdat[self.s])>0):
                self.nmrdat[self.s][self.e].autophase1d()
                
                
        # end autophase1d

    def autophase1dAll(self):
        nExp    = len(self.nmrdat[self.s])
        origExp = self.e
        for k in range(nExp):
            self.e = k
            self.autophase1d()
    
        self.e = origExp
        return "Finished autophase1dAll"
        # end autophase1dAll

    def autoref(self,tmsp=True):
        if(self.nmrdat[self.s][self.e].dim == 1):
            self.nmrdat[self.s][self.e].autoRef(tmsp)
            #self.nmrdat[self.s][self.e].setRef(np.array([0.0]), np.array([14836]))
            
        else:
            self.nmrdat[self.s][self.e].autoRef()
            #refShifts                = np.array([1.33, 19.3])
            #refPoints                = np.array([262, 2150])
            #self.nmrdat[self.s][self.e].setRef(refShifts, refPoints)
        
        return "Finished autoref"
        # end autoref

    def autorefAll(self):
        nExp    = len(self.nmrdat[self.s])
        origExp = self.e
        for k in range(nExp):
            self.e = k
            self.autoref()
        
        self.e = origExp
        return "Finished autorefAll"
        # end autorefAll
        
    def baseline1d(self):
        if(self.nmrdat[self.s][self.e].dim == 1):
            self.nmrdat[self.s][self.e].baseline1d()
            
        # end baseline1d
        
    def baseline1dAll(self):
        origExp = self.e
        for k in range(len(self.nmrdat[self.s])):
            self.e = k
            self.baseline1d()
            
        self.e = origExp
        # end baseline1dAll
        
    def bucketSpectra(self):
        idx1 = np.arange(len(self.nmrdat[self.s][0].ppm1))
        idx2 = idx1[::int(self.pp.bucketPoints)]
        idx2 = np.append(idx2, len(idx1))
        ppm  = np.array([])
        for k in range(len(idx2)-1):
            ppm = np.append(ppm, np.mean(self.nmrdat[self.s][0].ppm1[idx2[k]:idx2[k+1]]))
            
        for k in range(len(self.nmrdat[self.s])):
            spc = np.array([])
            for l in range(len(idx2)-1):
                spc = np.append(spc, np.sum(self.nmrdat[self.s][k].spc[0][idx2[l]:idx2[l+1]].real/self.pp.bucketPoints))
            
            self.nmrdat[self.s][k].ppm1   = ppm
            self.nmrdat[self.s][k].spc    = np.resize(self.nmrdat[self.s][k].spc,(1,len(spc)))
            self.nmrdat[self.s][k].spc[0] = np.copy(spc)
        
        # end bucketSpectra
        
    def clear(self):
        self.nmrdat = [[]]
        self.s = 0
        self.e = -1
        return "Workspace cleared"
        # end clear

    def compressBuckets(self):
        if (len(self.pp.compressStart) != len(self.pp.compressEnd)):
            return

        for k in range(len(self.pp.compressStart)):
            idx = np.where((self.nmrdat[self.s][0].ppm1 > self.pp.compressStart[k]) & (
                        self.nmrdat[self.s][0].ppm1 < self.pp.compressEnd[k]))
            self.deselect[idx] = np.ones(len(idx))
            self.deselect[int(np.round(np.mean(idx)))] = 0
            #print(idx)
            for l in range(len(self.nmrdat[self.s])):
                val = np.sum(self.nmrdat[self.s][l].spc[0][idx])
                self.nmrdat[self.s][l].spc[0][idx] = np.zeros(len(idx))
                self.nmrdat[self.s][l].spc[0][int(np.round(np.mean(idx)))] = val


        # end compressBuckets

    def dataPreProcessing(self):
        if(self.nmrdat[self.s][0].projectedJres == False):
            self.ftAll()
            self.baseline1dAll()
            self.autorefAll()
            self.shiftRef()

        else:
            s = self.s
            e = self.e
            self.s = self.nmrdat[s][e].origJresSet
            self.e = self.nmrdat[s][e].origJresExp
            self.pjres(s+1, self.nmrdat[s][e].pjresMode)
            self.s = s
            self.e = e

        self.noiseFilteringInit()
        self.deselect  = np.zeros(len(self.nmrdat[self.s][0].spc[0]))
        self.deselect2 = np.zeros(len(self.nmrdat[self.s][0].spc[0]))
        if(self.pp.flagExcludeRegion == True):
            self.excludeRegion()
            
        if(self.pp.flagSegmentalAlignment == True):
            self.segmentalAlignment()
        
        if(self.pp.flagNoiseFiltering == True):
            self.noiseFiltering()
            
        idx  = np.where(self.deselect  == 1)
        idx2 = np.where(self.deselect2 == len(self.nmrdat[self.s]))
        for k in range(len(self.nmrdat[self.s])):
            self.nmrdat[self.s][k].spc[0][idx]  = np.zeros(len(idx))
            self.nmrdat[self.s][k].spc[0][idx2] = np.zeros(len(idx2))
        
        for k in range(len(self.nmrdat[self.s])):
            self.nmrdat[self.s][k].spc[0][idx]  = np.zeros(len(idx))
            self.nmrdat[self.s][k].spc[0][idx2] = np.zeros(len(idx2))

        if(self.pp.flagBucketSpectra == True):
            self.bucketSpectra()
        
        if(self.pp.flagCompressBuckets == True):
            self.compressBuckets()
            
        if(self.pp.flagScaleSpectra == True):
            self.scaleSpectra()

        spc = np.zeros(len(self.nmrdat[self.s][0].spc[0]))
        nSpc = len(self.nmrdat[self.s])
        for k in range(nSpc):
            spc += self.nmrdat[self.s][k].spc[0].real

        idx = np.where(spc == 0)
        mVal = 0
        for k in range(len(self.nmrdat[self.s])):
            mVal = min(mVal,np.min(self.nmrdat[self.s][k].spc[0].real))

        if mVal < 0 and self.pp.avoidNegativeValues:
            for k in range(len(self.nmrdat[self.s])):
                self.nmrdat[self.s][k].spc[0] -= mVal

        for k in range(len(self.nmrdat[self.s])):
            self.nmrdat[self.s][k].spc[0][idx] = np.zeros(len(idx))

        if(self.pp.flagVarianceStabilisation == True):
            self.varianceStabilisation()
            
        if(self.pp.flagExportDataSet == True):
            self.exportDataSet()
        
        # end dataPreProcessing
        
    def excludeRegion(self):
        if(len(self.pp.excludeStart) != len(self.pp.excludeEnd)):
            return
        
        for k in range(len(self.pp.excludeStart)):
            idx                                = np.where((self.nmrdat[self.s][0].ppm1 > self.pp.excludeStart[k]) & (self.nmrdat[self.s][0].ppm1 < self.pp.excludeEnd[k]))
            self.deselect[idx]                 = np.ones(len(idx))
        
        # end excludeRegion
        
    def exportDataSet(self):
        if self.pp.exportMethod == 0:
            print("export Excel format")
            if os.path.isdir(self.pp.exportExcelPath) is False:
                os.makedirs(self.pp.exportExcelPath)

            fName = os.path.join(self.pp.exportExcelPath, self.pp.exportExcel)
            wb = Workbook()
            wsNMRdata = wb.active
            wsNMRdata.title = "NMR data"
            wsMetaData = wb.create_sheet("Metadata")
            spc = np.zeros(len(self.nmrdat[self.s][0].spc[0]))
            npts = len(self.nmrdat[self.s][0].spc[0])
            nSpc = len(self.nmrdat[self.s])
            for k in range(nSpc):
                spc += self.nmrdat[self.s][k].spc[0].real

            deselect = np.zeros(npts)
            idx = np.where(spc == 0)
            deselect[idx] = np.ones(len(idx))
            select = np.where(deselect == 0)
            if (self.pp.exportSamplesInRowsCols == 0):  # samples in rows
                colString = []
                for s in itertools.islice(self.iter_all_strings(), len(select[0]) + 2):
                    colString.append(s)

                wsNMRdata['A1'] = 'Name'
                wsNMRdata['B1'] = 'Class / ppm -->'
                for k in range(len(select[0])):
                    wsNMRdata[colString[k+2] + '1'] = str(self.nmrdat[self.s][0].ppm1[select[0][k]])

                for k in range(len(self.nmrdat[self.s])):
                    dse = os.path.split(self.nmrdat[self.s][k].origDataSet)
                    ds = os.path.split(dse[0])
                    wsNMRdata['A' + str(k+2)] = ds[1] + " " + dse[1]
                    wsNMRdata['B' + str(k+2)] = str(self.pp.classSelect[k])
                    for l in range(len(select[0])):
                        wsNMRdata[colString[l+2] + str(k+2)] = str(self.nmrdat[self.s][k].spc[0][select[0][l]].real)



            else:  # samples in cols
                wsNMRdata['A1'] = 'Name'
                wsNMRdata['A2'] = 'Class / ppm -v'
                colString = []
                for s in itertools.islice(self.iter_all_strings(), nSpc + 1):
                    colString.append(s)

                for k in range(len(self.nmrdat[self.s])):
                    dse = os.path.split(self.nmrdat[self.s][k].origDataSet)
                    ds = os.path.split(dse[0])
                    wsNMRdata[colString[k+1] + '1'] = ds[1] + " " + dse[1]
                    wsNMRdata[colString[k+1] + '2'] = str(self.pp.classSelect[k])

                for l in range(len(select[0])):
                    wsNMRdata['A' + str(l+3)] = str(self.nmrdat[self.s][0].ppm1[select[0][l]])
                    for k in range(len(self.nmrdat[self.s])):
                        wsNMRdata[colString[k+1] + str(l+3)] = str(self.nmrdat[self.s][k].spc[0][select[0][l]].real)



            wb.save(fName)

        elif self.pp.exportMethod == 1:
            print("export CSV format")
            if os.path.isdir(self.pp.exportPathName) is False:
                os.makedirs(self.pp.exportPathName)

            fName = os.path.join(self.pp.exportPathName, self.pp.exportFileName)
            f = open(fName, 'w')
            if (self.pp.exportDelimiterTab == True):
                delim = '\t'
            else:
                delim = self.pp.exportCharacter

            spc = np.zeros(len(self.nmrdat[self.s][0].spc[0]))
            for k in range(len(self.nmrdat[self.s])):
                spc += self.nmrdat[self.s][k].spc[0].real

            deselect = np.zeros(len(self.nmrdat[self.s][0].spc[0]))
            idx = np.where(spc == 0)
            deselect[idx] = np.ones(len(idx))
            if (self.pp.exportSamplesInRowsCols == 0):  # samples in rows
                f.write("ppm" + delim + " ")
                for k in range(len(self.nmrdat[self.s][0].ppm1)):
                    if (deselect[k] == 0):
                        f.write(delim + str(self.nmrdat[self.s][0].ppm1[k]))


                f.write("\n")
                for k in range(len(self.nmrdat[self.s])):
                    dse = os.path.split(self.nmrdat[self.s][k].origDataSet)
                    ds = os.path.split(dse[0])
                    f.write(ds[1] + " " + dse[1] + delim + self.pp.classSelect[k])
                    for l in range(len(self.nmrdat[self.s][k].spc[0])):
                        if (deselect[l] == 0):
                            f.write(delim + str(self.nmrdat[self.s][k].spc[0][l].real))


                    f.write("\n")


            else:  # samples in cols
                f.write("ppm")
                for k in range(len(self.nmrdat[self.s])):
                    dse = os.path.split(self.nmrdat[self.s][k].origDataSet)
                    ds = os.path.split(dse[0])
                    f.write(delim + ds[1] + " " + dse[1])

                f.write("\n")
                f.write(" ")
                for k in range(len(self.pp.classSelect)):
                    f.write(delim + self.pp.classSelect[k])

                f.write("\n")
                for k in range(len(self.nmrdat[self.s][0].spc[0])):
                    if (deselect[k] == 0):
                        f.write(str(self.nmrdat[self.s][0].ppm1[k]))
                        for l in range(len(self.nmrdat[self.s])):
                            f.write(delim + str(self.nmrdat[self.s][l].spc[0][k].real))

                        f.write("\n")



            f.close()

        elif self.pp.exportMethod == 2:
            print("export MetaboAnalyst")
            if os.path.isdir(self.pp.exportMetaboAnalystPath) is False:
                os.makedirs(self.pp.exportMetaboAnalystPath)

            fName = os.path.join(self.pp.exportMetaboAnalystPath, self.pp.exportMetaboAnalyst)
            f = open(fName, 'w')
            delim = ','
            spc = np.zeros(len(self.nmrdat[self.s][0].spc[0]))
            for k in range(len(self.nmrdat[self.s])):
                spc += self.nmrdat[self.s][k].spc[0].real

            deselect = np.zeros(len(self.nmrdat[self.s][0].spc[0]))
            idx = np.where(spc == 0)
            deselect[idx] = np.ones(len(idx))
            f.write("Sample" + delim + " Class")
            for k in range(len(self.nmrdat[self.s][0].ppm1)):
                if (deselect[k] == 0):
                    f.write(delim + " Bin." + str(self.nmrdat[self.s][0].ppm1[k]))


            f.write("\n")
            for k in range(len(self.nmrdat[self.s])):
                dse = os.path.split(self.nmrdat[self.s][k].origDataSet)
                ds = os.path.split(dse[0])
                f.write(ds[1] + " " + dse[1] + delim + " " + self.pp.classSelect[k])
                for l in range(len(self.nmrdat[self.s][k].spc[0])):
                    if (deselect[l] == 0):
                        f.write(delim + " " + str(self.nmrdat[self.s][k].spc[0][l].real))


                f.write("\n")


            f.close()

        elif self.pp.exportMethod == 3:
            print("export rDolphin")
            if os.path.isdir(self.pp.exportrDolphinPath) is False:
                os.makedirs(self.pp.exportrDolphinPath)

            fName = os.path.join(self.pp.exportrDolphinPath, self.pp.exportrDolphin)
            f = open(fName, 'w')
            delim = ','
            spc = np.zeros(len(self.nmrdat[self.s][0].spc[0]))
            for k in range(len(self.nmrdat[self.s])):
                spc += self.nmrdat[self.s][k].spc[0].real

            deselect = np.zeros(len(self.nmrdat[self.s][0].spc[0]))
            idx = np.where(spc == 0)
            deselect[idx] = np.ones(len(idx))
            f.write(str(self.nmrdat[self.s][0].ppm1[0]))
            for k in range(1, len(self.nmrdat[self.s][0].ppm1)):
                if (deselect[k] == 0):
                    f.write(delim + str(self.nmrdat[self.s][0].ppm1[k]))


            f.write("\n")
            for k in range(len(self.nmrdat[self.s])):
                dse = os.path.split(self.nmrdat[self.s][k].origDataSet)
                ds = os.path.split(dse[0])
                f.write(str(self.nmrdat[self.s][k].spc[0][0].real))
                for l in range(1, len(self.nmrdat[self.s][k].spc[0])):
                    if (deselect[l] == 0):
                        f.write(delim + str(self.nmrdat[self.s][k].spc[0][l].real))


                f.write("\n")

            f.close()
            fName = os.path.join(self.pp.exportrDolphinPath, "Parameters.csv")
            f = open(fName, 'w')
            f.write("Parameter,Value\n")
            f.write("nmr folder path,\n")
            f.write("1D data index,\n")
            f.write("proc_no,\n")
            f.write("spectra dataset path (csv format)," + self.pp.exportrDolphinPath + "/" + self.pp.exportrDolphin + "\n")
            f.write("Metadata path (csv format)," + self.pp.exportrDolphinPath + "/Metadata.csv\n")
            f.write("ROI patters file," + self.pp.exportrDolphinPath + "/ROI_profile.csv\n")
            f.write("Normalization (0=No;1=Eretic; 2=TSP; 3=Creatinine; 4=Spectra Sum; 5=PQN),2\n")
            f.write("Alignment (0=No;1=Glucose; 2=TSP; 3=Formate),2\n")
            f.write("Suppression,12-9.5;6.1-5.6;5.1-4.5\n")
            f.write("Spectrometer Frequency (MHz)," + str(self.nmrdat[self.s][0].acq.sfo1) + "\n")
            f.write("Bucket resolution," + str(self.pp.bucketPPM) + "\n")
            f.write("Biofluid,Urine\n")
            f.write("2D-Path,\n")
            f.write("Specific program parameters,\n")
            f.close()
            fName = os.path.join(self.pp.exportrDolphinPath, "Metadata.csv")
            f = open(fName, 'w')
            f.write("Sample,Individual,Sample Type\n")
            for k in range(len(self.nmrdat[self.s])):
                f.write(os.path.split(self.nmrdat[self.s][k].origDataSet)[1] + "," + str(k + 1) + ",1\n")

            f.close()

        elif self.pp.exportMethod == 4:
            print("export Batman")
            if os.path.isdir(self.pp.exportBatmanPath) is False:
                os.makedirs(self.pp.exportBatmanPath)

            fName = os.path.join(self.pp.exportBatmanPath, self.pp.exportBatman)
            f = open(fName, 'w')
            delim = '\t'
            spc = np.zeros(len(self.nmrdat[self.s][0].spc[0]))
            for k in range(len(self.nmrdat[self.s])):
                spc += self.nmrdat[self.s][k].spc[0].real

            deselect = np.zeros(len(self.nmrdat[self.s][0].spc[0]))
            idx = np.where(spc == 0)
            deselect[idx] = np.ones(len(idx))
            f.write("ppm")
            for k in range(len(self.nmrdat[self.s])):
                f.write(delim + "X" + str(k + 1))

            f.write("\n")
            for k in range(len(self.nmrdat[self.s][0].spc[0])):
                if (deselect[k] == 0):
                    f.write(str(self.nmrdat[self.s][0].ppm1[k]))
                    for l in range(len(self.nmrdat[self.s])):
                        f.write(delim + str(self.nmrdat[self.s][l].spc[0][k].real))

                    f.write("\n")


            f.close()

        else:
            print("export Bruker")
            if os.path.isdir(self.pp.exportBrukerPath + os.sep + self.pp.exportBruker) is False:
                os.makedirs(self.pp.exportBrukerPath + os.sep + self.pp.exportBruker)

            mmax = 0
            for k in range(len(self.nmrdat[self.s])):
                mmax = max(np.max(self.nmrdat[0][0].spc[0].real), mmax)

            scaleFactor = 2 * mmax / 2147483647
            for k in range(len(self.nmrdat[self.s])):
                self.nmrdat[self.s][k].exportBruker1d(self.pp.exportBrukerPath + os.sep + self.pp.exportBruker, str(k+1), scaleFactor)


    # end exportDataSet
        
    def ft(self):
        if(self.nmrdat[self.s][self.e].dim == 1):
            self.nmrdat[self.s][self.e].procSpc1D()

        else:
            self.nmrdat[self.s][self.e].procSpc()

        # end ft

    def ftAll(self):
        nExp    = len(self.nmrdat[self.s])
        origExp = self.e
        for k in range(nExp):
            self.e = k
            self.ft()
        
        self.e = origExp
        return "Finished ftAll"
        # end ftAll

    def help(self):
        fName = os.path.join(os.path.dirname(__file__), "web", "introduction", "index.html")
        url = "file://" + fName
        webbrowser.open(url, new=2)
        # end help

    def iter_all_strings(self):
        for size in itertools.count(1):
            for s in itertools.product(ascii_uppercase, repeat=size):
                yield "".join(s)

        # end iter_all_strings

    def load(self, dataSetName):
        dataSets = np.array([])
        lDir = os.listdir(dataSetName)
        curDataNotFound = True
        for k in range(len(lDir)):
            if (lDir[k] == 'curPars.dat'):
                curDataNotFound = False

        if (curDataNotFound == True):
            return

        fName = os.path.join(dataSetName, 'curPars.dat')
        f = open(fName, 'rb')
        curPars = pickle.load(f)
        f.close()
        self.fileFormatVersion = curPars[0]
        self.s = curPars[1]
        self.e = curPars[2]
        #self.pp = curPars[3]
        c = curPars[3]
        for k in self.pp.__dict__.keys():
            if hasattr(c, k):
                exec('self.pp.' + k + '=c.' + k)


        self.deselect = curPars[4]
        self.deselect2 = curPars[5]
        self.cmdBuffer = curPars[6]
        self.cmdIdx = curPars[7]
        self.script = curPars[8]
        self.console = curPars[9]
        for k in range(len(lDir)):
            if (os.path.isdir(os.path.join(dataSetName, lDir[k]))):
                dataSets = np.append(dataSets, lDir[k])

        dataSets = np.sort(dataSets)
        dataSetExps = []
        for k in range(len(dataSets)):
            dirName = os.listdir(os.path.join(dataSetName, dataSets[k]))
            dataExps = np.array([])
            for l in range(len(dirName)):
                if (os.path.isdir(os.path.join(os.path.join(dataSetName, dataSets[k]), dirName[l]))):
                    if (os.path.isfile(os.path.join(os.path.join(os.path.join(dataSetName, dataSets[k]), dirName[l]),
                                                    'titleFile.txt')) and
                            os.path.isfile(
                                os.path.join(os.path.join(os.path.join(dataSetName, dataSets[k]), dirName[l]),
                                             'acqusText.txt')) and
                            os.path.isfile(
                                os.path.join(os.path.join(os.path.join(dataSetName, dataSets[k]), dirName[l]),
                                             'acqu2sText.txt')) and
                            os.path.isfile(
                                os.path.join(os.path.join(os.path.join(dataSetName, dataSets[k]), dirName[l]),
                                             'acqu3sText.txt')) and
                            os.path.isfile(
                                os.path.join(os.path.join(os.path.join(dataSetName, dataSets[k]), dirName[l]),
                                             'procsText.txt')) and
                            os.path.isfile(
                                os.path.join(os.path.join(os.path.join(dataSetName, dataSets[k]), dirName[l]),
                                             'proc2sText.txt')) and
                            os.path.isfile(
                                os.path.join(os.path.join(os.path.join(dataSetName, dataSets[k]), dirName[l]),
                                             'proc3sText.txt')) and
                            os.path.isfile(
                                os.path.join(os.path.join(os.path.join(dataSetName, dataSets[k]), dirName[l]),
                                             'nmrDataSet.dat'))):
                        dataExps = np.append(dataExps, dirName[l])

            dataExps2 = []
            for k in range(len(dataExps)):
                try:
                    dataExps2.append(int(dataExps[k]))
                except:
                    pass

            dataExps2.sort()
            dataExps = list(map(str, dataExps2))
            dataSetExps.append(dataExps)

        self.nmrdat = []
        for k in range(len(dataSetExps)):
            self.nmrdat.append([])
            for l in range(len(dataSetExps[k])):
                fName = os.path.join(os.path.join(os.path.join(dataSetName, dataSets[k]), dataSetExps[k][l]),
                                     'nmrDataSet.dat')
                f = open(fName, 'rb')
                n = pickle.load(f)
                f.close()
                nd2 = nd.NmrData()
                for kk in nd2.__dict__.keys():
                    if kk is not 'acq' and kk is not 'proc' and kk is not 'disp' and kk is not 'apc':
                        if hasattr(n, kk):
                            exec('nd2.' + kk + '=n.' + kk)

                    elif kk is 'acq':
                        if hasattr(n, kk):
                            a = n.acq
                            aq = nd2.acq
                            for kkk in aq.__dict__.keys():
                                if kkk is not 'regEx':
                                    if hasattr(a, kkk):
                                        exec('aq.' + kkk + '=a.' + kkk)

                                else:
                                    r  = a.regEx
                                    re = aq.regEx
                                    for kkkk in re.__dict__.keys():
                                        if hasattr(r, kkkk):
                                            exec('re.' + kkkk + '=r.' + kkkk)

                                    aq.regEx = re


                            nd2.acq = aq


                    elif kk is 'proc':
                        if hasattr(n, kk):
                            p = n.proc
                            pc = nd2.proc
                            for kkk in pc.__dict__.keys():
                                if kkk is not 'regEx':
                                    if hasattr(p, kkk):
                                        exec('pc.' + kkk + '=p.' + kkk)
                                else:
                                    r = p.regEx
                                    re = pc.regEx
                                    for kkkk in re.__dict__.keys():
                                        if hasattr(r, kkkk):
                                            exec('re.' + kkkk + '=r.' + kkkk)

                                    aq.regEx = re


                        nd2.proc = pc


                    elif kk is 'disp':
                        if hasattr(n, kk):
                            d = n.disp
                            dp = nd2.disp
                            for kkk in dp.__dict__.keys():
                                if hasattr(d, kkk):
                                    exec('dp.' + kkk+ '=d.' + kkk)


                            nd2.disp = dp


                    elif kk is 'apc':
                        if hasattr(n, kk):
                            ab = n.apc
                            ac = nd2.apc
                            for kkk in ac.__dict__.keys():
                                if hasattr(ab, kkk):
                                    exec('ac.' + kkk + '=ab.' + kkk)


                            nd2.apc = ac



                self.nmrdat[k].append(nd2)
                nd2 = []
                fName = os.path.join(os.path.join(os.path.join(dataSetName, dataSets[k]), dataSetExps[k][l]),
                                     'titleFile.txt')
                f = open(fName, 'r')
                self.nmrdat[k][l].title = f.read()
                f.close()
                fName = os.path.join(os.path.join(os.path.join(dataSetName, dataSets[k]), dataSetExps[k][l]),
                                     'acqusText.txt')
                f = open(fName, 'r')
                self.nmrdat[k][l].acq.acqusText = f.read()
                f.close()
                fName = os.path.join(os.path.join(os.path.join(dataSetName, dataSets[k]), dataSetExps[k][l]),
                                     'acqu2sText.txt')
                f = open(fName, 'r')
                self.nmrdat[k][l].acq.acqu2sText = f.read()
                f.close()
                fName = os.path.join(os.path.join(os.path.join(dataSetName, dataSets[k]), dataSetExps[k][l]),
                                     'acqu3sText.txt')
                f = open(fName, 'r')
                self.nmrdat[k][l].acq.acqu3sText = f.read()
                f.close()
                fName = os.path.join(os.path.join(os.path.join(dataSetName, dataSets[k]), dataSetExps[k][l]),
                                     'procsText.txt')
                f = open(fName, 'r')
                self.nmrdat[k][l].proc.procsText = f.read()
                f.close()
                fName = os.path.join(os.path.join(os.path.join(dataSetName, dataSets[k]), dataSetExps[k][l]),
                                     'proc2sText.txt')
                f = open(fName, 'r')
                self.nmrdat[k][l].proc.proc2sText = f.read()
                f.close()
                fName = os.path.join(os.path.join(os.path.join(dataSetName, dataSets[k]), dataSetExps[k][l]),
                                     'proc3sText.txt')
                f = open(fName, 'r')
                self.nmrdat[k][l].proc.proc2sText = f.read()
                f.close()

        # end load

    def noiseFiltering(self):
        val    = self.pp.noiseThreshold*self.pp.stdVal
        for k in range(len(self.nmrdat[self.s])):
            idx =  np.where(self.nmrdat[self.s][k].spc[0].real < val)
            self.deselect2[idx] += np.ones(len(idx))
            if(False):
                idx2 = np.where(self.nmrdat[self.s][k].spc[0].real < 0)
                self.nmrdat[self.s][k].spc[0][idx2] = np.zeros(len(idx2))


        # end noiseFiltering
        
    def noiseFilteringInit(self):
        spcIdx         = np.where((self.nmrdat[self.s][0].ppm1>self.pp.noiseStart) & (self.nmrdat[self.s][0].ppm1<self.pp.noiseEnd))
        self.pp.stdVal = np.std(self.nmrdat[self.s][0].spc[0][spcIdx].real)
        # end noiseFilteringInit
        
    def pjres(self, set = -1, mode = 'skyline'):
        if(set<1):
            set = self.s + 2

        if(len(self.nmrdat)<set):
            self.nmrdat.append([])

        self.nmrdat[set-1] = []
        for k in range(len(self.nmrdat[self.s])):
            nd1 = nd.NmrData()
            self.nmrdat[set-1].append(nd1)
            self.nmrdat[set-1][k].dim = 1
            self.nmrdat[set-1][k].fid = [[]]
            self.nmrdat[set-1][k].acq = self.nmrdat[self.s][k].acq
            self.nmrdat[set-1][k].proc = self.nmrdat[self.s][k].proc
            self.nmrdat[set-1][k].disp = self.nmrdat[self.s][k].disp
            self.nmrdat[set-1][k].ppm1 = np.resize(self.nmrdat[set-1][k].ppm1, (len(self.nmrdat[self.s][k].ppm1)))
            self.nmrdat[set-1][k].ppm1 = np.copy(self.nmrdat[self.s][k].ppm1)
            self.nmrdat[set-1][k].spc = np.resize(self.nmrdat[set-1][k].spc, (1, len(self.nmrdat[self.s][k].spc[0])))
            self.nmrdat[set-1][k].refShift = self.nmrdat[self.s][k].refShift
            self.nmrdat[set-1][k].refPoint = self.nmrdat[self.s][k].refPoint
            self.nmrdat[set-1][k].title = "pJres spectrum\n" + self.nmrdat[self.s][k].title
            self.nmrdat[set-1][k].pjresMode = mode
            self.nmrdat[set-1][k].projectedJres = True
            self.nmrdat[set-1][k].origJresSet = self.s
            self.nmrdat[set-1][k].origJresExp = k
            if(mode == 'skyline'):
                self.nmrdat[set-1][k].spc[0] = np.max(self.nmrdat[self.s][k].spc,0)
            else:
                self.nmrdat[set-1][k].spc[0] = np.sum(self.nmrdat[self.s][k].spc,0)


        # end jresProject

    def plotSpc(self):
        pl.plot()
        ax = pl.gca()
        xlim = ax.get_xlim()
        ylim = ax.get_ylim()
        ax.clear()
        if (len(self.nmrdat[self.s]) == 0):
            return

        if (len(self.nmrdat[self.s][self.e].spc) == 0):
            return

        d = self.nmrdat[self.s][self.e].disp
        if (d.posCol == "RGB"):
            posCol = d.posColRGB
        else:
            posCol = d.posCol

        if (d.negCol == "RGB"):
            negCol = d.negColRGB
        else:
            negCol = d.negCol

        posCol = matplotlib.colors.to_hex(posCol)
        negCol = matplotlib.colors.to_hex(negCol)
        xlabel = d.xLabel + " [" + d.axisType1 + "]"
        ylabel = d.yLabel + " [" + d.axisType2 + "]"
        if (self.nmrdat[self.s][self.e].dim == 1):
            for k in range(len(self.nmrdat[self.s])):
                if ((k != self.e) and (self.nmrdat[self.s][k].disp.displaySpc == True)):
                    d = self.nmrdat[self.s][k].disp
                    if (d.posCol == "RGB"):
                        posCol = d.posColRGB
                    else:
                        posCol = d.posCol

                    if (d.negCol == "RGB"):
                        negCol = d.negColRGB
                    else:
                        negCol = d.negCol

                    posCol = matplotlib.colors.to_hex(posCol)
                    negCol = matplotlib.colors.to_hex(negCol)
                    pl.plot(self.nmrdat[self.s][k].ppm1, self.nmrdat[self.s][k].spc[0].real, color=posCol)

            d = self.nmrdat[self.s][self.e].disp
            if (d.posCol == "RGB"):
                posCol = d.posColRGB
            else:
                posCol = d.posCol

            if (d.negCol == "RGB"):
                negCol = d.negColRGB
            else:
                negCol = d.negCol

            posCol = matplotlib.colors.to_hex(posCol)
            negCol = matplotlib.colors.to_hex(negCol)
            xlabel = d.xLabel + " [" + d.axisType1 + "]"
            ylabel = d.yLabel + " [" + d.axisType2 + "]"
            pl.plot(self.nmrdat[self.s][self.e].ppm1, self.nmrdat[self.s][self.e].spc[0].real, color=posCol)
            ax = pl.gca()
            ax.set_xlabel(xlabel)
            ax.autoscale()
            ax.invert_xaxis()
            if (self.keepZoom == True):
                if xlim[0] != -0.05 and xlim[1] != 1.05 and ylim[0] != -0.05 and ylim[1] != 1.05:
                    ax.set_xlim(xlim)
                    ax.set_ylim(ylim)


        else:
            mm = np.max(np.abs(self.nmrdat[self.s][self.e].spc.real))
            posLev = np.linspace(d.minLevel * mm, d.maxLevel * mm, d.nLevels)
            negLev = np.linspace(-d.maxLevel * mm, -d.minLevel * mm, d.nLevels)
            pl.contour(self.nmrdat[self.s][self.e].ppm1,
                                                 self.nmrdat[self.s][self.e].ppm2,
                                                 self.nmrdat[self.s][self.e].spc.real, posLev, colors=posCol,
                                                 linestyles='solid', antialiased=True)
            pl.contour(self.nmrdat[self.s][self.e].ppm1,
                                                 self.nmrdat[self.s][self.e].ppm2,
                                                 self.nmrdat[self.s][self.e].spc.real, negLev, colors=negCol,
                                                 linestyles='solid', antialiased=True)
            ax = pl.gca()
            ax.set_xlabel(xlabel)
            ax.set_ylabel(ylabel)
            ax.autoscale()
            ax.invert_xaxis()
            ax.invert_yaxis()

        # end plotSpc

    def preProcInit(self):
        self.pp.init(len(self.nmrdat[self.s]))
        # end preProcInit
        
    def readSpc(self, dataSetName, dataSetNumber):
        self.e            = len(self.nmrdat[self.s])
        nd1               = nd.NmrData()
        nd1.dataSetName   = dataSetName
        nd1.dataSetNumber = dataSetNumber
        nd1.readSpc()
        self.nmrdat[self.s].append(nd1)
        # end readSpc

    def readSpcs(self, dataPath, dataExp):
        if len(dataExp) > 1:
            for k in range(len(dataExp)):
                self.readSpc(dataPath[0], str(dataExp[k]))


        else:
            for k in range(len(dataPath)):
                self.readSpc(dataPath[k], str(dataExp[0]))


    # end readSpcs

    def readNMRPipeSpc(self, dataSetName, dataSetNumber, procDataName = 'test.dat'):
        self.e            = len(self.nmrdat[self.s])
        nd1               = nd.NmrData()
        nd1.dataSetName   = dataSetName
        nd1.dataSetNumber = dataSetNumber
        nd1.readSpc()
        nd1.readPipe2D(dataSetName + os.sep + dataSetNumber + '.proc', procDataName)
        self.nmrdat[self.s].append(nd1)
        # end readSpc

    def readNMRPipeSpcs(self, dataPath, dataExp, procDataName = 'test.dat'):
        if len(dataExp) > 1:
            for k in range(len(dataExp)):
                self.readNMRPipeSpc(dataPath[0], str(dataExp[k]), procDataName)


        else:
            for k in range(len(dataPath)):
                self.readNMRPipeSpc(dataPath[k], str(dataExp[0]), procDataName)


    # end readSpcs

    def resetDataPreProcessing(self):
        if not self.nmrdat[self.s][0].projectedJres:
            self.ftAll()
            self.baseline1dAll()
            self.autorefAll()
            self.shiftRef()
        else:
            s = self.s
            e = self.e
            self.s = self.nmrdat[s][e].origJresSet
            self.e = self.nmrdat[s][e].origJresExp
            self.pjres(s + 1, self.nmrdat[s][e].pjresMode)
            self.s = s
            self.e = e

        # end dataPreProcessing

    def save(self,dataSetName):
        if(len(dataSetName) == 0):
            return
        
        try:
            os.makedirs(dataSetName)
        except:
            pass
        
        fName = os.path.join(dataSetName,'curPars.dat')
        f     = open(fName,'wb')
        pickle.dump([self.fileFormatVersion, self.s, self.e, self.pp, self.deselect, self.deselect2, self.cmdBuffer, self.cmdIdx, self.script, self.console], f)
        f.close()
        for k in range(len(self.nmrdat)):
            setPath = os.path.join(dataSetName,str(k+1))
            try:
                os.makedirs(setPath)
            except:
                pass
            
            for l in range(len(self.nmrdat[k])):
                expPath = os.path.join(setPath,str(l+1))
                try:
                    os.makedirs(expPath)
                except:
                    pass
                
                fName   = os.path.join(expPath,'nmrDataSet.dat')
                f       = open(fName,'wb')
                pickle.dump(self.nmrdat[k][l],f)
                f.close()
                fName   = os.path.join(expPath,'titleFile.txt')
                f       = open(fName,'w')
                f.write(self.nmrdat[k][l].title)
                f.close()
                fName   = os.path.join(expPath, 'procsText.txt')
                f       = open(fName,'w')
                f.write(self.nmrdat[k][l].proc.procsText)
                f.close()
                fName   = os.path.join(expPath, 'proc2sText.txt')
                f       = open(fName,'w')
                f.write(self.nmrdat[k][l].proc.proc2sText)
                f.close()
                fName   = os.path.join(expPath, 'proc3sText.txt')
                f       = open(fName,'w')
                f.write(self.nmrdat[k][l].proc.proc3sText)
                f.close()
                fName   = os.path.join(expPath, 'acqusText.txt')
                f       = open(fName,'w')
                f.write(self.nmrdat[k][l].acq.acqusText)
                f.close()
                fName   = os.path.join(expPath, 'acqu2sText.txt')
                f       = open(fName,'w')
                f.write(self.nmrdat[k][l].acq.acqu2sText)
                f.close()
                fName   = os.path.join(expPath, 'acqu3sText.txt')
                f       = open(fName,'w')
                f.write(self.nmrdat[k][l].acq.acqu3sText)
                f.close()
        # end save
        
    def scaleSpectra(self):
        nSpc     = len(self.nmrdat[self.s])
        npts     = len(self.nmrdat[self.s][0].spc[0])
        if self.pp.scaleSpectraRefSpc > 0:
            refSpc = self.nmrdat[self.s][self.pp.scaleSpectraRefSpc - 1].spc[0].real
        else:
            refSpcs = np.zeros((nSpc, npts))
            for k in range(nSpc):
                refSpcs[k] = self.nmrdat[self.s][k].spc[0].real

            if self.pp.segAlignRefSpc == 0:
                refSpc = np.mean(refSpcs, 0)
            else:
                refSpc = np.median(refSpcs, 0)

            refSpcs = np.array([[]])

        scale = np.ones(nSpc)
        if self.pp.scalePQN is True:
            for k in range(nSpc):
                scaleVect = self.nmrdat[self.s][k].spc[0][np.where(refSpc != 0)].real/refSpc[np.where(refSpc != 0)]
                self.nmrdat[self.s][k].spc[0] /= np.median(scaleVect[np.where(scaleVect != 0)])
        else:
            if self.pp.preserveOverallScale is True:
                for k in range(nSpc):
                    scale[k] = np.sum(self.nmrdat[self.s][k].spc[0]).real

            for k in range(nSpc):
                self.nmrdat[self.s][k].spc[0] /= np.sum(self.nmrdat[self.s][k].spc[0]).real
                self.nmrdat[self.s][k].spc[0] *= np.max(scale)


        # end scaleSpectra

    def segmentalAlignment(self):
        segStart = self.nmrdat[self.s][0].ppm2points(self.pp.segStart, 0)
        segEnd   = self.nmrdat[self.s][0].ppm2points(self.pp.segEnd,   0)
        npts     = len(self.nmrdat[self.s][0].spc[0])
        nSpc     = len(self.nmrdat[self.s])
        excludeStart = np.zeros((len(segStart), nSpc))
        excludeEnd = np.zeros((len(segStart), nSpc))
        if self.pp.segAlignRefSpc > 0:
            refSpc = self.nmrdat[self.s][self.pp.segAlignRefSpc - 1].spc[0]
        else:
            refSpcs = np.zeros((nSpc, npts))
            for k in range(nSpc):
                refSpcs[k] = self.nmrdat[self.s][k].spc[0].real

            if self.pp.segAlignRefSpc == 0:
                refSpc = np.mean(refSpcs, 0)
            else:
                refSpc = np.median(refSpcs, 0)

            refSpcs = np.array([[]])

        posShift = np.zeros((nSpc, len(segStart)))
        negShift = np.zeros((nSpc, len(segStart)))
        for k in range(nSpc):
            if k != self.pp.segAlignRefSpc - 1:
                for l in range(len(segStart)):
                    startPts = npts - segEnd[l]
                    endPts   = npts - segStart[l]
                    corrSpc1 = np.copy(refSpc[startPts:endPts].real)
                    corrSpc2 = self.nmrdat[self.s][k].spc[0][startPts:endPts].real
                    maxShift = len(corrSpc1) - 1
                    zeros = np.zeros(len(corrSpc1))
                    corrSpc1 = np.append(np.append(zeros, corrSpc1), zeros)
                    corrSpc2 = np.append(np.append(zeros, corrSpc2), zeros)
                    shifts = np.linspace(-maxShift, maxShift, 2 * maxShift + 1, dtype='int')
                    corrVect = np.zeros(2*maxShift + 1)
                    spcShift = 0
                    for m in shifts:
                        corrVect[m + maxShift] = np.corrcoef(corrSpc1, np.roll(corrSpc2, m))[0][1]

                    maxCorrShifts = shifts[np.where(corrVect == np.max(corrVect))]
                    minShift = np.where(np.abs(maxCorrShifts) == np.min(np.abs(maxCorrShifts)))
                    if np.max(corrVect) > 0.8:
                        spcShift = maxCorrShifts[minShift][0]
                        corrSpc2 = self.nmrdat[self.s][k].spc[0][startPts:endPts]
                        corrSpc2 = np.append(np.append(zeros, corrSpc2), zeros)
                        corrSpc2 = np.roll(corrSpc2, spcShift)[maxShift + 1:2*maxShift + 2]
                        self.nmrdat[self.s][k].spc[0][startPts:endPts] = np.copy(corrSpc2)
                        exSta = 0
                        exEnd = 0
                        if spcShift < 0:
                            negShift[k][l] = 0 - spcShift
                            exEnd = self.nmrdat[self.s][self.pp.segAlignRefSpc - 1].points2ppm(npts - endPts - spcShift, 0)
                            exSta = self.nmrdat[self.s][self.pp.segAlignRefSpc - 1].points2ppm(npts - endPts, 0)
                        elif spcShift > 0:
                            posShift[k][l] = spcShift
                            exEnd = self.nmrdat[self.s][self.pp.segAlignRefSpc - 1].points2ppm(npts - startPts, 0)
                            exSta = self.nmrdat[self.s][self.pp.segAlignRefSpc - 1].points2ppm(npts - startPts - spcShift, 0)

                        excludeStart[l][k] = exSta
                        excludeEnd[l][k] = exEnd





        ps = np.max(posShift, 0)
        ns = np.max(negShift, 0)
        ps2 = np.transpose(posShift)
        ns2 = np.transpose(negShift)
        excludeStart2 = np.array([])
        excludeEnd2   = np.array([])
        for k in range(len(posShift[0])):
            l = np.where(ps2[k] == ps[k])[0][0]
            if ps[k] > 0:
                sVal = math.floor(1e4 * excludeStart[k][l]) / 1e4
                eVal = math.floor(1e4 * excludeEnd[k][l]) / 1e4
                if sVal not in self.pp.excludeStart and eVal not in self.pp.excludeEnd:
                    excludeStart2 = np.append(excludeStart2, math.floor(1e4 * sVal) / 1e4)
                    excludeEnd2   = np.append(excludeEnd2,   math.floor(1e4 * eVal) / 1e4)
                    #self.pp.excludeStart = np.append(self.pp.excludeStart, sVal)
                    #self.pp.excludeEnd   = np.append(self.pp.excludeEnd,   eVal)

            l = np.where(ns2[k] == ns[k])[0][0]
            if ns[k] > 0:
                sVal = math.floor(1e4 * excludeStart[k][l]) / 1e4
                eVal = math.floor(1e4 * excludeEnd[k][l]) / 1e4
                if sVal not in self.pp.excludeStart and eVal not in self.pp.excludeEnd:
                    excludeStart2 = np.append(excludeStart2, math.floor(1e4 * sVal) / 1e4)
                    excludeEnd2   = np.append(excludeEnd2,   math.floor(1e4 * eVal) / 1e4)
                    #self.pp.excludeStart = np.append(self.pp.excludeStart, math.floor(1e4 * sVal) / 1e4)
                    #self.pp.excludeEnd = np.append(self.pp.excludeEnd, math.floor(1e4 * eVal) / 1e4)



        #self.excludeRegion()
        if self.pp.segAlignRefSpc > 0:
            spcIdx = self.pp.segAlignRefSpc - 1
        else:
            spcIdx = 0

        for k in range(len(excludeStart2)):
            idx = np.where((self.nmrdat[self.s][spcIdx].ppm1 > excludeStart2[k]) & (
                        self.nmrdat[self.s][spcIdx].ppm1 < excludeEnd2[k]))
            self.deselect[idx] = np.ones(len(idx))

    # end segmentalAlignment

    def selectPlotAll(self):
        for k in range(len(self.nmrdat[self.s])):
            self.nmrdat[self.s][k].disp.displaySpc = True

        self.plotSpc()
        # end selectPlotAll

    def selectPlotClear(self):
        for k in range(len(self.nmrdat[self.s])):
            self.nmrdat[self.s][k].disp.displaySpc = False

        self.plotSpc()
        # end selectPlotClear

    def setGb(self, gb):
        nExp = len(self.nmrdat[self.s])
        for k in range(nExp):
            for l in range(len(gb)):
                self.nmrdat[self.s][k].proc.gb[l] = gb[l]
            
        
    
        return "setGb"
    # end setGb
        
    def setLb(self, lb):
        nExp = len(self.nmrdat[self.s])
        for k in range(nExp):
            for l in range(len(lb)):
                self.nmrdat[self.s][k].proc.lb[l] = lb[l]
            
        
    
        return "setLb"
    # end setLb
        
    def setPhFromExp(self, exp = -1):
        if(exp == -1):
            exp = self.e
            
        nExp = len(self.nmrdat[self.s])
        for k in range(nExp):
            if(k != exp):
                self.nmrdat[self.s][k].proc.ph0 = self.nmrdat[self.s][exp].proc.ph0
                self.nmrdat[self.s][k].proc.ph1 = self.nmrdat[self.s][exp].proc.ph1
                
        
        return "setPhFromExp"
    # end setPhFromExp
        
    def setPh0(self, ph0):
        nExp = len(self.nmrdat[self.s])
        for k in range(nExp):
            for l in range(len(ph0)):
                self.nmrdat[self.s][k].proc.ph0[l] = ph0[l]
            
        
    
        return "setPh0"
    # end setPh0
        
    def setPh1(self, ph1):
        nExp = len(self.nmrdat[self.s])
        for k in range(nExp):
            for l in range(len(ph1)):
                self.nmrdat[self.s][k].proc.ph1[l] = ph1[l]
            
        
    
        return "setPh1"
    # end setPh1
        
    def setSsb(self, ssb):
        nExp = len(self.nmrdat[self.s])
        for k in range(nExp):
            for l in range(len(ssb)):
                self.nmrdat[self.s][k].proc.ssb[l] = ssb[l]
            
        
    
        return "setSsb"
    # end setSsb
        
    def setWindowType(self, wt):
        nExp = len(self.nmrdat[self.s])
        for k in range(nExp):
            for l in range(len(wt)):
                self.nmrdat[self.s][k].proc.windowType[l] = wt[l]
            
        
    
        return "setWindowType"
    # end setWindowType
        
    def setZeroFill(self, zf):
        nExp = len(self.nmrdat[self.s])
        for k in range(nExp):
            for l in range(len(zf)):
                self.nmrdat[self.s][k].proc.nPoints[l] = zf[l]
            
        
    
        return "setZeroFill"
    # end setZeroFill
    
    def shiftRef(self):
        for k in range(len(self.nmrdat[self.s])):
            shiftDelta = self.nmrdat[self.s][k].refPoint[0] - self.nmrdat[self.s][0].refPoint[0]
            self.nmrdat[self.s][k].spc[0] = np.roll(self.nmrdat[self.s][k].spc[0],shiftDelta)
            self.nmrdat[self.s][k].ppm1   = self.nmrdat[self.s][0].ppm1
            self.nmrdat[self.s][k].refShift = self.nmrdat[self.s][0].refShift
    # end shiftRef

    def varianceStabilisation(self):
        if self.pp.autoScaling:
            self.varStabAutoScale()

        if self.pp.paretoScaling:
            self.varStabParetoScale()

        if self.pp.gLogTransform:
            self.varStabgLogTransform()

    # end varianceStabilisation

    def varStabAutoScale(self):
        npts = len(self.nmrdat[self.s][0].spc[0])
        nSpc = len(self.nmrdat[self.s])
        spcs = np.zeros((nSpc, npts))
        for k in range(nSpc):
            spcs[k] = self.nmrdat[self.s][k].spc[0].real

        spcMean = np.mean(spcs, 0)
        for k in range(nSpc):
            spcs[k] -= spcMean

        spcVar = np.var(spcs, 0)
        for k in range(nSpc):
            idx = np.where(spcVar != 0)
            spcs[k][idx] = spcs[k][idx]/np.sqrt(spcVar[idx])
            self.nmrdat[self.s][k].spc[0] = spcs[k]

    # end varStabAutoScale

    def varStabgLogTransform(self):
        nSpc = len(self.nmrdat[self.s])
        lMin = np.zeros(nSpc)
        lMax = np.zeros(nSpc)
        for k in range(nSpc):
            lMax[k] = np.max(self.nmrdat[self.s][k].spc[0].real)

        mMax = np.max(lMax)
        for k in range(nSpc):
            spc = np.copy(self.nmrdat[self.s][k].spc[0].real)
            spc /= mMax
            spc = spc - self.pp.varY0 + np.sqrt((spc -self.pp.varY0)**2 + self.pp.varLambda)
            idx = np.where(spc <= 0)
            spc[idx] = 1e-100
            self.nmrdat[self.s][k].spc[0] = np.copy(np.log(spc))
            lMin[k] = np.min(self.nmrdat[self.s][k].spc[0].real)

        for k in range(nSpc):
            self.nmrdat[self.s][k].spc[0] -= np.min(lMin)

    # end varStabgLogTransform

    def varStabParetoScale(self):
        npts = len(self.nmrdat[self.s][0].spc[0])
        nSpc = len(self.nmrdat[self.s])
        spcs = np.zeros((nSpc, npts))
        for k in range(nSpc):
            spcs[k] = self.nmrdat[self.s][k].spc[0].real

        spcVar = np.sqrt(np.std(spcs, 0))
        for k in range(nSpc):
            idx = np.where(spcVar != 0)
            spcs[k][idx] = spcs[k][idx]/spcVar[idx]
            self.nmrdat[self.s][k].spc[0] = spcs[k]

    # end varStabParetoScale



'''
NMR spectrum processing parameters
'''

import numpy as np
import os
from metabolabpy.nmr import procRegEx
from metabolabpy.nmr import procProcparRegEx
import math

class ProcPars:
    
    def __init__(self):
        self.procsText             = str('')
        self.proc2sText            = str('')
        self.proc3sText            = str('')
        self.tilt                  = False
        self.symj                  = False
        self.ph0                   = np.array([0.0, 0.0, 0.0])
        self.ph1                   = np.array([0.0, 0.0, 0.0])
        self.phCorr                = np.array([1, 1, 0], dtype = 'int')
        self.refShift              = np.array([0.0, 0.0, 0.0])
        self.refPoint              = np.array([0, 0, 0])
        self.nPoints               = np.array([0, 0, 0])
        self.pivot                 = np.array([0, 0, 0])
        self.lb                    = np.array([0.3, 0.0, 0.0])
        self.gb                    = np.array([0.0, 0.0, 0.0])
        self.ssb                   = np.array([0.0, 0.0, 0.0])
        self.axisNucleus           = np.array(['      ','      ','      '], dtype = 'str')
        self.aunmp                 = str('')
        self.polyOrder             = 4
        self.waterSuppression      = 0
        self.gibbs                 = np.array([True, True, False])
        self.convExtrapolationSize = np.array([32, 0, 0])
        self.convWindowSize        = np.array([32, 0, 0])
        self.windowType            = np.array([1, 4, 0], dtype = 'int')
        self.convWindowType        = np.array([0, 0, 0], dtype = 'int')
        self.regEx                 = procRegEx.ProcRegEx()
        self.regExVarian           = procProcparRegEx.ProcProcparRegEx()
        self.sw_h                  = np.array([0.0, 0.0, 0.0])
        self.sf                    = np.array([0.0, 0.0, 0.0])
        self.fidOffsetCorrection   = 0
        self.stripStart            = 0
        self.stripEnd              = 0
        self.phaseInversion        = True
        self.multFactor            = [0, 0]
        self.windowFunctions       = {
            0             : "None",
            1             : "Exponential",
            2             : "Gaussian",
            3             : "Sine",
            4             : "QSine",
            5             : "SEM",
            "None"        : 0,
            "Exponential" : 1,
            "Gaussian"    : 2,
            "Sine"        : 3,
            "QSine"       : 4,
            "SEM"         : 5
        }
        self.phaseCorrection       = {
            0           : "None",
            1           : "Manual",
            2           : "Automatic",
            "None"      : 0,
            "Manual"    : 1,
            "Automatic" : 2
        }
        self.winType = {
            0          : "Gaussian",
            1          : "Sine",
            "Gaussian" : 0,
            "Sine"     : 1
        }
        self.gibbsP = {
            0     : False,
            1     : True,
            False : 0,
            True  : 1
        }
        self.waterSupp = {
            0         : "None",
            1         : "Conv",
            2         : "Poly",
            3         : "Wavewat",
            "None"    : 0,
            "Conv"    : 1,
            "Poly"    : 2,
            "Wavewat" : 3
        }
        self.dataType = ''
        # end __init__

    def __str__(self): # pragma: no cover
        return self.procsText
        # end __str__
        
    def parseRegEx(self):
        self.ph0[0]         = float(self.regEx.ph0.findall(self.procsText)[0][0])
        self.ph1[0]         = float(self.regEx.ph1.findall(self.procsText)[0][0])
        self.nPoints[0]     = int(self.regEx.nPoints.findall(self.procsText)[0])
        self.lb[0]          = float(self.regEx.lb.findall(self.procsText)[0][0])
        self.gb[0]          = float(self.regEx.gb.findall(self.procsText)[0][0])
        self.ssb[0]         = float(self.regEx.ssb.findall(self.procsText)[0][0])
        self.sf[0]          = float(self.regEx.sf.findall(self.procsText)[0][0])
        #self.stripStart     = int(self.regEx.stsr.findall(self.procsText)[0][0])
        #self.stripEnd       = int(self.regEx.stsi.findall(self.procsText)[0][0])
        if self.ssb[0] == 2.0:
            self.ssb[0] = 90.0

        if self.procsText.find("$AXNUC=") > -1:
            self.axisNucleus[0] = self.regEx.axisNucleus.findall(self.procsText)[0]
            self.axisNucleus[0] = self.axisNucleus[0].replace('<', '')
            self.axisNucleus[0] = self.axisNucleus[0].replace('>', '')

        if self.proc2sText.find("$AXNUC=") > -1:
            self.axisNucleus[1] = self.regEx.axisNucleus.findall(self.proc2sText)[0]
            self.axisNucleus[1] = self.axisNucleus[1].replace('<', '')
            self.axisNucleus[1] = self.axisNucleus[1].replace('>', '')

        self.windowType[0]  = int(self.regEx.wdw.findall(self.procsText)[0])
        self.aunmp          = self.regEx.aunmp.findall(self.procsText)[0]
        try:
            self.sf[1] = float(self.regEx.sf.findall(self.proc2sText)[0][0])
        except:
            pass

        try:
            self.ph0[1]         = float(self.regEx.ph0.findall(self.proc2sText)[0][0])
            self.ph1[1]         = float(self.regEx.ph1.findall(self.proc2sText)[0][0])
            self.nPoints[1]     = int(self.regEx.nPoints.findall(self.proc2sText)[0])
            self.lb[1]          = float(self.regEx.lb.findall(self.proc2sText)[0][0])
            self.gb[1]          = float(self.regEx.gb.findall(self.proc2sText)[0][0])
            self.ssb[1]         = float(self.regEx.ssb.findall(self.proc2sText)[0][0])
            if self.ssb[1] == 2.0:
                self.ssb[1] = 90.0

            self.wdw[1]         = float(self.regEx.wdw.findall(self.proc2sText)[0][0])
            #self.axisNucleus[1] = self.regEx.axisNucleus.findall(self.proc2sText)[0]
            self.windowType[1]  = int(self.regEx.wdw.findall(self.proc2sText)[0])

        except:
            a = 2+3
            #print("1D data")
        
        # end parseRegEx

    def parseRegExVarian(self):
        dd        = self.regExVarian.td.search(self.procsText)
        if hasattr(dd, 'span'):
            dd        = self.procsText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            td        = int(dd)
            self.nPoints[0] = pow(2, math.ceil(math.log(td)/math.log(2)))

        ni = 1
        dd        = self.regExVarian.ni.search(self.procsText)
        if hasattr(dd, 'span'):
            dd        = self.procsText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            ni = int(dd)

        ni = max(ni,1)
        ni2 = 1
        dd        = self.regExVarian.ni2.search(self.procsText)
        if hasattr(dd, 'span'):
            dd        = self.procsText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            ni2 = int(dd)

        ni2 = max(ni2,1)
        phase = np.array([1], dtype='int')
        dd        = self.regExVarian.phase.search(self.procsText)
        if hasattr(dd, 'span'):
            dd         = self.procsText[dd.span()[0] + 1:]
            dd         = dd[dd.find('\n') + 1:]
            dd         = dd[dd.find(' '):dd.find('\n')]
            phase = np.array(dd.split(), dtype='int')

        phase2 = np.array([1], dtype='int')
        dd        = self.regExVarian.phase2.search(self.procsText)
        if hasattr(dd, 'span'):
            dd         = self.procsText[dd.span()[0] + 1:]
            dd         = dd[dd.find('\n') + 1:]
            dd         = dd[dd.find(' '):dd.find('\n')]
            phase2 = np.array(dd.split(), dtype='int')

        self.nPoints[1] = pow(2, math.ceil(math.log(ni*ni2*len(phase)*len(phase2))/math.log(2)))
        dd        = self.regExVarian.nuc1.search(self.procsText)
        if hasattr(dd, 'span'):
            dd        = self.procsText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            dd        = dd.replace(" ", "")
            dd        = dd.replace("\"", "")
            self.axisNucleus[0] = dd

        dd        = self.regExVarian.nuc2.search(self.procsText)
        if hasattr(dd, 'span'):
            dd        = self.procsText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            dd        = dd.replace(" ", "")
            dd        = dd.replace("\"", "")
            self.axisNucleus[1] = dd

        dd        = self.regExVarian.nuc3.search(self.procsText)
        if hasattr(dd, 'span'):
            dd        = self.procsText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            dd        = dd.replace(" ", "")
            dd        = dd.replace("\"", "")
            self.axisNucleus[2] = dd

        # end parseRegExVarian

    def read(self, spcDir):
        procsName    = spcDir + os.sep + 'pdata' + os.sep + '1' + os.sep + 'procs'
        proc2sName   = spcDir + os.sep + 'pdata' + os.sep + '1' + os.sep + 'proc2s'
        proc3sName   = spcDir + os.sep + 'pdata' + os.sep + '1' + os.sep + 'proc3s'
        procparName  = spcDir + os.sep + 'procpar'
        procparName2 = spcDir + os.sep + 'PROCPAR'
        if(os.path.isfile(procsName)):
            try:
                f = open(procsName, "r")
                self.procsText = f.read()
                f.close()

            except:
                f = open(procsName, "r", encoding='latin-1')
                self.procsText = f.read()
                f.close()

            self.dataType = 'Bruker'

        if(os.path.isfile(proc2sName)):
            try:
                f = open(proc2sName,"r")
                self.proc2sText = f.read()
                f.close()

            except:
                f = open(proc2sName,"r", encoding='latin-1')
                self.proc2sText = f.read()
                f.close()

            self.dataType = 'Bruker'

        if(os.path.isfile(proc3sName)):
            try:
                f = open(proc3sName,"r")
                self.proc3sText = f.read()
                f.close()

            except:
                f = open(proc3sName,"r", encoding='latin-1')
                self.proc3sText = f.read()
                f.close()

            self.dataType = 'Bruker'

        if (os.path.isfile(procparName)):
            f = open(procparName, "r")
            self.procsText = f.read()
            f.close()
            self.dataType = 'Varian'

        if (os.path.isfile(procparName2)):
            f = open(procparName2, "r")
            self.procsText = f.read()
            f.close()
            self.dataType = 'Varian'

        if self.dataType == 'Bruker':
            self.parseRegEx()
        else:
            self.parseRegExVarian()

        # end read
    



'''
NMR spectrum acquisition parameters
'''

import numpy as np
import os
from metabolabpy.nmr import acqRegEx
from metabolabpy.nmr import acqProcparRegEx

class AcqPars:
    
    def __init__(self):
        self.acqusText        = str('')
        self.acqu2sText       = str('')
        self.acqu3sText       = str('')
        self.byteOrder        = -1
        self.sw               = np.array([0, 0, 0],dtype = 'float64')
        self.sw_h             = np.array([0, 0, 0],dtype = 'float64')
        self.sfo1             = 0.0
        self.sfo2             = 0.0
        self.sfo3             = 0.0
        self.bf1              = 0.0
        self.bf2              = 0.0
        self.bf3              = 0.0
        self.o1               = 0.0
        self.o2               = 0.0
        self.o3               = 0.0
        self.nDataPoints      = np.array([0, 0, 0],dtype = 'int')
        self.aqMode           = 0
        self.decim            = 0
        self.dspfvs           = 0
        self.groupDelay       = 0.0
        self.digMod           = 0
        self.transients       = 0
        self.steadyStateScans = 0
        self.relaxationDelay  = 0.0
        self.spinRate         = 0.0
        self.nndp             = 0.0
        self.pulseProgram     = str('')
        self.pulProgName      = str('')
        self.instrument       = str('')
        self.dataType         = 0
        self.solvent          = str('')
        self.probe            = str('')
        self.title            = str('')
        self.origin           = str('')
        self.owner            = str('')
        self.metaInfo         = str('')
        self.aunm             = str('')
        self.temperature      = -273.15
        self.cnst             = np.array([], 'float64')
        self.delay            = np.array([], 'float64')
        self.pulse            = np.array([], 'float64')
        self.pcpd             = np.array([], 'float64')
        self.powerLevel       = np.array([], 'float64')
        self.powerLevelWatt   = np.array([], 'float64')
        self.powerLevelMax    = np.array([], 'float64')
        self.shapedPower      = np.array([], 'float64')
        self.shapedPowerWatt  = np.array([], 'float64')
        self.spoal            = np.array([], 'float64')
        self.spoffs           = np.array([], 'float64')
        self.cpdProg          = np.array([], dtype = 'str')
        self.gpName           = np.array([], dtype = 'str')
        self.vcList           = str('')
        self.vdList           = str('')
        self.vpList           = str('')
        self.vaList           = str('')
        self.vtList           = str('')
        self.nuc1             = str('')
        self.nuc2             = str('')
        self.nuc3             = str('')
        self.nuc4             = str('')
        self.nuc5             = str('')
        self.nuc6             = str('')
        self.nuc7             = str('')
        self.nuc8             = str('')
        self.gpx              = np.array([], dtype = 'float64')
        self.gpy              = np.array([], dtype = 'float64')
        self.gpz              = np.array([], dtype = 'float64')
        self.increments       = np.array([], dtype = 'float64')
        self.nusList          = str('')
        self.nusAmount        = 0.0
        self.nusSeed          = 0
        self.nusJsp           = 0
        self.nusT2            = 0.0
        self.nusTD            = 0
        self.overFlow         = 0
        self.pynm             = str('')
        self.spcFrequency     = np.array([0, 0], dtype = 'float64')
        self.spcSFreq         = np.array([0, 0], dtype = 'float64')
        self.spcNucleus       = np.array(['     ', '     '], dtype = 'str')
        self.spcOffset        = np.array([0, 0], dtype = 'float64')
        self.acqT0            = np.array([0, 0], dtype = 'int')
        self.fnMode           = 0
        self.inf              = np.array([], dtype = 'float64')
        self.regEx            = acqRegEx.AcqRegEx()
        self.regExVarian      = acqProcparRegEx.AcqProcparRegEx()
        self.manufacturer     = ''
        self.ni               = 0
        self.ni2              = 0
        self.np               = 1
        self.np2              = 1
        self.phase            = np.array([], dtype = 'int')
        self.phase2           = np.array([], dtype = 'int')
        self.autopos          = ''
        # end __init__
        
    def __str__(self): # pragma: no cover
        return self.acqusText
        # end __str__
        
    def parseRegEx(self):
        self.sfo1             = float(self.regEx.sfo1.findall(self.acqusText)[0])
        self.sfo2             = float(self.regEx.sfo2.findall(self.acqusText)[0])
        self.sfo3             = float(self.regEx.sfo3.findall(self.acqusText)[0])
        self.sfo4             = float(self.regEx.sfo4.findall(self.acqusText)[0])
        self.sfo5             = float(self.regEx.sfo5.findall(self.acqusText)[0])
        self.sfo6             = float(self.regEx.sfo6.findall(self.acqusText)[0])
        self.sfo7             = float(self.regEx.sfo7.findall(self.acqusText)[0])
        self.sfo8             = float(self.regEx.sfo8.findall(self.acqusText)[0])
        self.bf1              = float(self.regEx.bf1.findall(self.acqusText)[0])
        self.bf2              = float(self.regEx.bf2.findall(self.acqusText)[0])
        self.bf3              = float(self.regEx.bf3.findall(self.acqusText)[0])
        self.bf4              = float(self.regEx.bf4.findall(self.acqusText)[0])
        self.bf5              = float(self.regEx.bf5.findall(self.acqusText)[0])
        self.bf6              = float(self.regEx.bf6.findall(self.acqusText)[0])
        self.bf7              = float(self.regEx.bf7.findall(self.acqusText)[0])
        self.bf8              = float(self.regEx.bf8.findall(self.acqusText)[0])
        self.o1               = float(self.regEx.o1.findall(self.acqusText)[0])
        self.o2               = float(self.regEx.o2.findall(self.acqusText)[0])
        self.o3               = float(self.regEx.o3.findall(self.acqusText)[0])
        self.o4               = float(self.regEx.o4.findall(self.acqusText)[0])
        try:
            self.o5               = float(self.regEx.o5.findall(self.acqusText)[0])
            self.o6               = float(self.regEx.o6.findall(self.acqusText)[0])
            self.o7               = float(self.regEx.o7.findall(self.acqusText)[0])
            self.o8               = float(self.regEx.o8.findall(self.acqusText)[0])
        except:
            pass
        
        self.sw[0]            = float(self.regEx.sw.findall(self.acqusText)[0])
        self.sw_h[0]          = float(self.regEx.sw_h.findall(self.acqusText)[0])
        self.nDataPoints[0]   = int(self.regEx.td.findall(self.acqusText)[0])
        self.decim            = int(self.regEx.decim.findall(self.acqusText)[0])
        self.dspfvs           = int(self.regEx.dspfvs.findall(self.acqusText)[0])
        if self.acqusText.find("$GRPDLY=") > -1:
            self.groupDelay       = float(self.regEx.grpdly.findall(self.acqusText)[0][0])
            if self.groupDelay < 0:
                self.groupDelay = 0

        self.byteOrder        = int(self.regEx.byteOrder.findall(self.acqusText)[0])
        self.aqMode           = int(self.regEx.aqMode.findall(self.acqusText)[0])
        self.digMod           = int(self.regEx.digMod.findall(self.acqusText)[0])
        self.transients       = int(self.regEx.transients.findall(self.acqusText)[0])
        self.steadyStateScans = int(self.regEx.steadyStateScans.findall(self.acqusText)[0])
        self.relaxationDelay  = float(self.regEx.relaxationDelay.findall(self.acqusText)[0]) if (len(self.regEx.relaxationDelay.findall(self.acqusText))>0) else 0.0
        self.spinRate         = int(self.regEx.spinRate.findall(self.acqusText)[0])
        self.pulProgName      = self.regEx.pulProg.findall(self.acqusText)[0]
        self.aunm             = self.regEx.aunm.findall(self.acqusText)[0]
        self.autopos          = self.regEx.autopos.findall(self.acqusText)[0]
        self.instrument       = self.regEx.instrument.findall(self.acqusText)[0]
        self.dataType         = int(self.regEx.dataType.findall(self.acqusText)[0])
        self.solvent          = self.regEx.solvent.findall(self.acqusText)[0]
        self.probe            = self.regEx.probe.findall(self.acqusText)[0]
        self.probe            = self.probe.replace('<','')
        self.probe            = self.probe.replace('>','')
        self.title            = self.regEx.title.findall(self.acqusText)[0]
        self.origin           = self.regEx.origin.findall(self.acqusText)[0]
        self.owner            = self.regEx.owner.findall(self.acqusText)[0]
        try:
            self.metaInfo         = self.regEx.metaInfo.findall(self.acqusText)[0]
        except:
            pass

        self.temperature      = float(self.regEx.temperature.findall(self.acqusText)[0])
        dd                    = self.regEx.cnst.search(self.acqusText)
        dd                    = self.acqusText[dd.span()[0]:]
        dd                    = dd[dd.find('\n')+1:]
        dd                    = dd[:dd.find('##$')]
        self.cnst             = np.array(dd.split(), dtype = 'float64')
        dd                    = self.regEx.delay.search(self.acqusText)
        dd                    = self.acqusText[dd.span()[0]:]
        dd                    = dd[dd.find('\n')+1:]
        dd                    = dd[:dd.find('##$')]
        self.delay            = np.array(dd.split(), dtype = 'float64')
        try:
            dd                    = self.regEx.cpdProg.search(self.acqusText)
            dd                    = self.acqusText[dd.span()[0]:]
            dd                    = dd[dd.find('\n')+1:]
            dd                    = dd[:dd.find('##$')]
            self.cpdProg          = np.array(dd.split(), dtype = 'str')
        except:
            pass
            
        try:
            dd                    = self.regEx.gpName.search(self.acqusText)
            dd                    = self.acqusText[dd.span()[0]:]
            dd                    = dd[dd.find('\n')+1:]
            dd                    = dd[:dd.find('##$')]
            self.gpName           = np.array(dd.split(), dtype = 'str')
        except:
            pass
            
        dd                    = self.regEx.gpx.search(self.acqusText)
        dd                    = self.acqusText[dd.span()[0]:]
        dd                    = dd[dd.find('\n')+1:]
        dd                    = dd[:dd.find('##$')]
        self.gpx              = np.array(dd.split(), dtype = 'float64')
        dd                    = self.regEx.gpy.search(self.acqusText)
        dd                    = self.acqusText[dd.span()[0]:]
        dd                    = dd[dd.find('\n')+1:]
        dd                    = dd[:dd.find('##$')]
        self.gpy              = np.array(dd.split(), dtype = 'float64')
        dd                    = self.regEx.gpz.search(self.acqusText)
        dd                    = self.acqusText[dd.span()[0]:]
        dd                    = dd[dd.find('\n')+1:]
        dd                    = dd[:dd.find('##$')]
        self.gpz              = np.array(dd.split(), dtype = 'float64')
        dd                    = self.regEx.increments.search(self.acqusText)
        dd                    = self.acqusText[dd.span()[0]:]
        dd                    = dd[dd.find('\n')+1:]
        dd                    = dd[:dd.find('##$')]
        self.increments       = np.array(dd.split(), dtype = 'float64')
        dd                    = self.regEx.pulse.search(self.acqusText)
        dd                    = self.acqusText[dd.span()[0]:]
        dd                    = dd[dd.find('\n')+1:]
        dd                    = dd[:dd.find('##$')]
        self.pulse            = np.array(dd.split(), dtype = 'float64')
        dd                    = self.regEx.pcpd.search(self.acqusText)
        dd                    = self.acqusText[dd.span()[0]:]
        dd                    = dd[dd.find('\n')+1:]
        dd                    = dd[:dd.find('##$')]
        self.pcpd            = np.array(dd.split(), dtype = 'float64')
        dd                    = self.regEx.powerLevel.search(self.acqusText)
        dd                    = self.acqusText[dd.span()[0]:]
        dd                    = dd[dd.find('\n')+1:]
        dd                    = dd[:dd.find('##$')]
        self.powerLevel       = np.array(dd.split(), dtype = 'float64')
        if self.acqusText.find("$PLW=") > -1:
            dd                    = self.regEx.powerLevelWatt.search(self.acqusText)
            dd                    = self.acqusText[dd.span()[0]:]
            dd                    = dd[dd.find('\n')+1:]
            dd                    = dd[:dd.find('##$')]
            self.powerLevelWatt   = np.array(dd.split(), dtype = 'float64')
            dd                    = self.regEx.powerLevelMax.search(self.acqusText)
            dd                    = self.acqusText[dd.span()[0]:]
            dd                    = dd[dd.find('\n')+1:]
            dd                    = dd[:dd.find('##$')]
            self.powerLevelMax    = np.array(dd.split(), dtype = 'float64')

        dd                    = self.regEx.shapedPower.search(self.acqusText)
        dd                    = self.acqusText[dd.span()[0]:]
        dd                    = dd[dd.find('\n')+1:]
        dd                    = dd[:dd.find('##$')]
        self.shapedPower      = np.array(dd.split(), dtype = 'float64')
        if self.acqusText.find("$SPW=") > -1:
            dd                    = self.regEx.shapedPowerWatt.search(self.acqusText)
            dd                    = self.acqusText[dd.span()[0]:]
            dd                    = dd[dd.find('\n')+1:]
            dd                    = dd[:dd.find('##$')]
            self.shapedPowerWatt  = np.array(dd.split(), dtype = 'float64')

        dd                    = self.regEx.spoal.search(self.acqusText)
        dd                    = self.acqusText[dd.span()[0]:]
        dd                    = dd[dd.find('\n')+1:]
        dd                    = dd[:dd.find('##$')]
        self.spoal            = np.array(dd.split(), dtype = 'float64')
        dd                    = self.regEx.spoffs.search(self.acqusText)
        dd                    = self.acqusText[dd.span()[0]:]
        dd                    = dd[dd.find('\n')+1:]
        dd                    = dd[:dd.find('##$')]
        self.spoffs           = np.array(dd.split(), dtype = 'float64')
        if self.acqusText.find("$INF=") > -1:
            dd                    = self.regEx.inf.search(self.acqusText)
            dd                    = self.acqusText[dd.span()[0]:]
            dd                    = dd[dd.find('\n')+1:]
            dd                    = dd[:dd.find('##$')]
            self.inf              = np.array(dd.split(), dtype = 'float64')

        self.vcList           = self.regEx.vcList.findall(self.acqusText)[0]
        self.vdList           = self.regEx.vdList.findall(self.acqusText)[0]
        self.vpList           = self.regEx.vpList.findall(self.acqusText)[0]
        self.vaList           = self.regEx.vaList.findall(self.acqusText)[0]
        self.vtList           = self.regEx.vtList.findall(self.acqusText)[0]
        self.nuc1             = self.regEx.nuc1.findall(self.acqusText)[0]
        self.nuc2             = self.regEx.nuc2.findall(self.acqusText)[0]
        self.nuc3             = self.regEx.nuc3.findall(self.acqusText)[0]
        self.nuc4             = self.regEx.nuc4.findall(self.acqusText)[0]
        self.nuc5             = self.regEx.nuc5.findall(self.acqusText)[0]
        self.nuc6             = self.regEx.nuc6.findall(self.acqusText)[0]
        self.nuc7             = self.regEx.nuc7.findall(self.acqusText)[0]
        self.nuc8             = self.regEx.nuc8.findall(self.acqusText)[0]
        if self.acqusText.find("$NUSLIST=") > -1:
            self.nusList          = self.regEx.nusList.findall(self.acqusText)[0]
            try:
                self.nusAmount        = float(self.regEx.nusAmount.findall(self.acqusText)[0])
                self.nusSeed          = int(self.regEx.nusSeed.findall(self.acqusText)[0])
                self.nusJsp           = int(self.regEx.nusJsp.findall(self.acqusText)[0])
                self.nusT2            = float(self.regEx.nusT2.findall(self.acqusText)[0][0])
            except:
                pass


        self.overFlow         = int(self.regEx.overFlow.findall(self.acqusText)[0])
        if self.acqusText.find("$PYNM=") > -1:
            self.pynm             = self.regEx.pynm.findall(self.acqusText)[0]

        self.spcFrequency[0]  = float(self.regEx.bf1.findall(self.acqusText)[0])
        self.spcSFreq[0]      = float(self.regEx.sfo1.findall(self.acqusText)[0])
        self.spcNucleus[0]    = self.regEx.nuc1.findall(self.acqusText)[0]
        self.spcOffset[0]     = float(self.regEx.o1.findall(self.acqusText)[0])
        try:
            self.sw[1]            = float(self.regEx.sw.findall(self.acqu2sText)[0])
        except:
            pass

        try:
            self.sw_h[1]          = float(self.regEx.sw_h.findall(self.acqu2sText)[0])
        except:
            pass

        try:
            self.spcFrequency[1]  = float(self.regEx.bf1.findall(self.acqu2sText)[0])
        except:
            pass

        try:
            self.spcSFreq[1]      = float(self.regEx.sfo1.findall(self.acqu2sText)[0])
        except:
            pass

        try:
            self.spcNucleus[1]    = self.regEx.nuc1.findall(self.acqu2sText)[0]
        except:
            pass

        try:
            self.spcOffset[1]     = float(self.regEx.o1.findall(self.acqu2sText)[0])
        except:
            pass

        try:
            self.acqT0[1]         = int(self.regEx.acqT0.findall(self.acqu2sText)[0])
        except:
            pass

        try:
            self.nDataPoints[1]   = int(self.regEx.td.findall(self.acqu2sText)[0])
        except:
            pass

        try:
            self.nusTD            = int(self.regEx.nusTD.findall(self.acqu2sText)[0])
        except:
            pass

        try:
            self.fnMode           = int(self.regEx.fnMode.findall(self.acqu2sText)[0])
        except:
            pass

        try:
            self.acqT0[0]         = int(self.regEx.acqT0.findall(self.acqusText)[0])            
        except:
            pass

        try:
            self.sw[2]            = float(self.regEx.sw.findall(self.acqu3sText)[0]) if (len(self.regEx.sw.findall(self.acqu3sText))>0) else 0.0
            self.sw_h[2]          = float(self.regEx.sw_h.findall(self.acqu3sText)[0]) if (len(self.regEx.sw_h.findall(self.acqu3sText))>0) else 0.0
            self.nDataPoints[2]   = int(self.regEx.td.findall(self.acqu3sText)[0]) if (len(self.regEx.td.findall(self.acqu3sText))>0) else 0

        except:
            pass
        
        # end parseRegEx

    def parseRegExVarian(self):
        dd        = self.regExVarian.sfo1.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            self.sfo1 = float(dd)

        dd        = self.regExVarian.sfo2.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            self.sfo2 = float(dd)

        dd        = self.regExVarian.sfo3.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            self.sfo3 = float(dd)

        dd        = self.regExVarian.o1.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            self.o1 = float(dd)

        dd        = self.regExVarian.o2.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            self.o2 = float(dd)

        dd        = self.regExVarian.o3.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            self.o3 = float(dd)

        self.bf1 = self.sfo1 - self.o1/1000.0
        self.bf2 = self.sfo2 - self.o2/1000.0
        self.bf3 = self.sfo3 - self.o3/1000.0
        self.o1 = 0
        self.o2 = 0
        self.o3 = 0
        dd        = self.regExVarian.sw_h.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            self.sw_h[0] = float(dd)
            self.sw[0]   = self.sw_h[0]/self.sfo1

        dd        = self.regExVarian.sw2_h.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            self.sw_h[1] = float(dd)
            self.sw[1]   = self.sw_h[1]/self.sfo2

        dd        = self.regExVarian.sw3_h.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            dd2 = self.regExVarian.sw2_h.search(self.acqusText)
            if hasattr(dd2, 'span'):
                self.sw_h[2] = float(dd)
                self.sw[2]   = self.sw_h[2]/self.sfo3
            else:
                self.sw_h[1] = float(dd)
                self.sw[1]   = self.sw_h[1]/self.sfo3


        dd        = self.regExVarian.td.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            self.nDataPoints[0] = int(dd)  # int(int(dd)/2)

        dd        = self.regExVarian.phase.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd      = self.acqusText[dd.span()[0] + 1:]
            dd      = dd[dd.find('\n') + 1:]
            dd      = dd[:dd.find(' ')]
            self.np = int(dd)

        dd        = self.regExVarian.phase2.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd       = self.acqusText[dd.span()[0] + 1:]
            dd       = dd[dd.find('\n') + 1:]
            dd       = dd[:dd.find(' ')]
            self.np2 = int(dd)

        dd        = self.regExVarian.ni.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            self.ni = int(dd)

        self.ni = max(self.ni, 1)
        dd        = self.regExVarian.ni2.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            self.ni2 = int(dd)

        self.ni2 = max(self.ni2, 1)
        dd        = self.regExVarian.phase.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd         = self.acqusText[dd.span()[0] + 1:]
            dd         = dd[dd.find('\n') + 1:]
            dd         = dd[dd.find(' '):dd.find('\n')]
            self.phase = np.array(dd.split(), dtype = 'int')

        dd        = self.regExVarian.phase2.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd         = self.acqusText[dd.span()[0] + 1:]
            dd         = dd[dd.find('\n') + 1:]
            dd         = dd[dd.find(' '):dd.find('\n')]
            self.phase2 = np.array(dd.split(), dtype = 'int')

        dd        = self.regExVarian.transients.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            self.transients = int(dd)

        dd        = self.regExVarian.steadyStateScans.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            self.steadyStateScans = int(dd)

        dd        = self.regExVarian.nucleus.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            dd        = dd.replace(" ", "")
            dd        = dd.replace("\"", "")
            self.spcNucleus = dd

        dd        = self.regExVarian.pulProgName.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            dd        = dd.replace(" ", "")
            dd        = dd.replace("\"", "")
            self.pulProgName = dd

        dd        = self.regExVarian.temperature.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            self.temperature = float(dd) + 273.15

        dd        = self.regExVarian.nuc1.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            dd        = dd.replace(" ", "")
            dd        = dd.replace("\"", "")
            self.nuc1 = dd

        dd        = self.regExVarian.nuc2.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            dd        = dd.replace(" ", "")
            dd        = dd.replace("\"", "")
            self.nuc2 = dd

        dd        = self.regExVarian.nuc3.search(self.acqusText)
        if hasattr(dd, 'span'):
            dd        = self.acqusText[dd.span()[0] + 1:]
            dd        = dd[dd.find('\n') + 1:]
            dd        = dd[dd.find(' '):dd.find('\n')]
            dd        = dd.replace(" ", "")
            dd        = dd.replace("\"", "")
            self.nuc3 = dd

        # end parseRegExVarian

    def read(self, spcDir):
        acqusName    = spcDir + os.sep + 'acqus'
        acqu2sName   = spcDir + os.sep + 'acqu2s'
        acqu3sName   = spcDir + os.sep + 'acqu3s'
        procparName  = spcDir + os.sep + 'procpar'
        procparName2 = spcDir + os.sep + 'PROCPAR'
        if(os.path.isfile(acqusName)):
            try:
                f = open(acqusName,"r")
                self.acqusText = f.read()
                f.close()

            except:
                f = open(acqusName,"r", encoding='latin-1')
                self.acqusText = f.read()
                f.close()

            self.manufacturer = 'Bruker'
            
        if(os.path.isfile(acqu2sName)):
            try:
                f = open(acqu2sName,"r")
                self.acqu2sText = f.read()
                f.close()

            except:
                f = open(acqu2sName,"r", encoding='latin-1')
                self.acqu2sText = f.read()
                f.close()

            self.manufacturer = 'Bruker'


        if(os.path.isfile(acqu3sName)):
            try:
                f = open(acqu3sName,"r")
                self.acqu3sText = f.read()
                f.close()

            except:
                f = open(acqu3sName,"r", encoding='latin-1')
                self.acqu3sText = f.read()
                f.close()

            self.manufacturer = 'Bruker'

        if (os.path.isfile(procparName)):
            f = open(procparName, "r")
            self.acqusText = f.read()
            f.close()
            self.byteOrder = 0
            self.manufacturer = 'Varian'

        if (os.path.isfile(procparName2)):
            f = open(procparName2, "r")
            self.acqusText = f.read()
            f.close()
            self.byteOrder = 0
            self.manufacturer = 'Varian'

        if self.manufacturer == 'Bruker':
            self.parseRegEx()
            if(self.groupDelay == 0.0):
                self.setGroupDelay()


        if self.manufacturer == 'Varian':
            self.parseRegExVarian()
            self.nDataPoints[1] = self.ni*self.ni2*len(self.phase)*len(self.phase2)*2
            self.groupDelay = 0.0
            self.decim = 0
            self.dspfvs = 0

        # end read
    
    def setGroupDelay(self):
        decims = np.array([2, 3, 4, 6, 8, 12, 16, 24, 32, 48, 64, 96, 128, 192, 256, 384, 512, 768, 1024, 1536, 2048])
        dspfirm10 = np.array([179, 201, 533, 709, 1097, 1449, 2225, 2929, 4481, 5889, 8993, 11809, 18017, 23649, 36065, 47329, 72161, 94689, 144353, 189409, 288737])
        dspfirm11 = np.array([184, 219, 384, 602, 852, 1668, 2312, 3368, 4656, 6768, 9344, 13568, 18560, 27392, 36992, 55040, 73856, 110336, 147584, 220928, 295040])
        dspfirm12 = np.array([184, 219, 384, 602, 852, 1668, 2292, 3368, 4616, 6768, 9264, 13568, 18560, 27392, 36992, 55040, 73856, 110336, 147584, 220928, 295040])
        dspfirm13 = np.array([11, 17, 23, 35, 47, 71, 95, 143, 191, 287, 383, 575])
        dspfirm14 = np.array([60, 90, 118, 179, 244, 360, 492, 724, 980, 1444, 1958, 2886, 3912, 5768, 7820, 11532])
        dspfirm15 = np.array([0, 0, 58, 152, 202, 318, 418, 642, 842, 1290, 1690, 2586, 3386])
        dspfirm = [dspfirm10, dspfirm11, dspfirm12, dspfirm13, dspfirm14, dspfirm15]
        addr = np.where(decims == self.decim)[0][0]
        dly = dspfirm[self.dspfvs-10][addr]
        if self.decim == 3 and self.dspfvs== 15 and self.sw_h > 104000.0:
            self.groupDelay = 55.0
        else:
            self.groupDelay = dly / self.decim / 2.0

        # end setGroupDelay





import os
import configparser


class NmrConfig:

    def __init__(self):
        self.autoPlot = True
        self.keepZoom = True
        self.fontSize = 13
        self.phaseReferenceColour = 'Red'
        self.fName = '.pyMetaboLab.config'
        self.homeDir = os.path.expanduser('~')
        self.configFile = os.path.join(self.homeDir, self.fName)
        self.value = ""

    def makeConfig(self):
        config = configparser.ConfigParser()
        autoPlot = 'yes' if self.autoPlot is True else 'no'
        keepZoom = 'yes' if self.keepZoom is True else 'no'
        config['GUI'] = {'autoPlot': autoPlot,
                         'keepZoom': keepZoom,
                         'fontSize': str(self.fontSize)}
        config['Disp'] = {'phaseReferenceColour': self.phaseReferenceColour}
        return config

    def saveConfig(self):
        config = self.makeConfig()
        with open(self.configFile, 'w') as configfile:
            config.write(configfile)

    def readConfig(self):
        config = configparser.ConfigParser()
        config.read(self.configFile)
        if len(config.sections()) == 0:
            config = self.makeConfig()
            self.saveConfig()

        for k in config.sections():
            for l in config[k]:
                self.setValues(l, config[k][l])


    def set_autoplot(self, value):
        self.autoPlot = True if value == "yes" else False

    def set_keepzoom(self, value):
        self.keepZoom = True if value == "yes" else False

    def set_fontsize(self, value):
        self.fontSize = float(value)

    def set_phasereferencecolour(self, value):
        self.phaseReferenceColour = value

    def setValues(self, key, value):
        mName = "self.set_" + key
        eval(mName)(value)




'''
Auomatic phase and baseline correction for 1D NMR spectra

Using the histogram method
'''

from metabolabpy.nmr import nmrData
import numpy as np
#import matplotlib.pyplot as pl
from scipy.optimize import fmin
import math
from scipy.fftpack import fft, ifft, fftshift


class Apcbc:

    def __init__(self):
        self.correctBaseline = 0
        self.npts = 0
        self.nMax = 10.00
        self.nOrder = 1
        self.nbins = 5001
        self.mFact0 = 20 * 10 ** (self.nOrder + 1)
        self.mFact1 = 5 * 10 ** (self.nOrder + 1)
        self.startPts = 3500
        self.endPts = 4000
        self.pars = np.array([])
        self.rSpc = np.array([0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0])
        self.iSpc = np.array([0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0])

    def baselineFitFunc(self, par, spc, xaxis):
        l = self.calcLine(spc, xaxis)
        spc = spc - l
        nc = int((len(par)) / 2)
        cr = par[:nc]
        ci = par[nc:]
        blr = np.polynomial.chebyshev.chebval(xaxis, cr)
        bli = np.polynomial.chebyshev.chebval(xaxis, ci)
        spc = spc - blr - 1j * bli
        hr = self.hist(spc.real)
        hi = self.hist(spc.imag)
        n1 = -self.qr(hr)
        n2 = self.qi(hi)
        # print("%4.2f & %4.2f" % (n1, n2))
        return 1.0 * n1 + 100.0 * n2
        # end baselineFitFunc

    def baselineFitFuncEval(self, par, spc, xaxis):  # , plot):
        l = self.calcLine(spc, xaxis)
        spc = spc - l
        nc = int((len(par)) / 2)
        cr = par[:nc]
        ci = par[nc:]
        blr = np.polynomial.chebyshev.chebval(xaxis, cr)
        bli = np.polynomial.chebyshev.chebval(xaxis, ci)
        spc = spc - blr - 1j * bli
        # if(plot == True):
        #    hr  = self.hist(spc.real)
        #    hi  = self.hist(spc.imag)
        #    xr  = hr[1][1:nbins+1]
        #    xi  = hi[1][1:nbins+1]
        #    pl.subplot(2,1,1)
        #    pl.plot(xr, hr[0])
        #    pl.subplot(2,1,2)
        #    pl.plot(xi, hi[0])
        #    pl.show()

        return spc
        # end baselineFitFuncEval

    def calcLine(self, spc, xaxis):
        x1 = np.mean(xaxis[self.startPts:self.endPts])
        x2 = np.mean(xaxis[-self.endPts:-self.startPts])
        y1r = np.mean(spc.real[self.startPts:self.endPts])
        y2r = np.mean(spc.real[-self.endPts:-self.startPts])
        y1i = np.mean(spc.imag[self.startPts:self.endPts])
        y2i = np.mean(spc.imag[-self.endPts:-self.startPts])
        slopeR = (y2r - y1r) / (x2 - x1)
        slopeI = (y2i - y1i) / (x2 - x1)
        lineR = slopeR * (xaxis - x1) + y1r
        lineI = slopeI * (xaxis - x1) + y1i
        line = lineR + 1j * lineI
        return line
        # end calcLine

    def fitBaseline(self, spc, xaxis):
        npts = len(spc)
        l = self.calcLine(spc, xaxis)
        nCoeffReal = np.polynomial.chebyshev.chebfit(xaxis, spc.real - l.real, self.nOrder)
        nCoeffImag = np.polynomial.chebyshev.chebfit(xaxis, spc.imag - l.imag, self.nOrder)
        pars = np.array([])
        pars = np.append(pars, nCoeffReal)
        pars = np.append(pars, nCoeffImag)
        print("...Fitting phase and baseline...")
        plsq = fmin(self.baselineFitFunc, pars, args=(spc, xaxis), ftol=1e-3, xtol=1e-3, maxfun=1000)
        print("...finished")
        rSpcPar = plsq[:int(len(plsq) / 2)]
        iSpcPar = plsq[int(len(plsq) / 2):]
        self.rSpc[:len(rSpcPar)] = rSpcPar
        self.iSpc[:len(iSpcPar)] = iSpcPar
        return plsq
        # end fitBaseline

    def fitPhase(self, spc, xaxis):
        npts = len(spc)
        ph0 = 0.0
        ph1 = 0.0
        l = self.calcLine(spc, xaxis)
        nCoeffReal = np.polynomial.chebyshev.chebfit(xaxis, spc.real - l.real, self.nOrder)
        nCoeffImag = np.polynomial.chebyshev.chebfit(xaxis, spc.imag - l.imag, self.nOrder)
        pars = np.array([])
        pars = np.append(pars, ph0)
        pars = np.append(pars, ph1)
        pars = np.append(pars, nCoeffReal)
        pars = np.append(pars, nCoeffImag)
        print("...Fitting phase and baseline...")
        plsq = fmin(self.phaseFitFunc, pars, args=(spc, xaxis), ftol=1e-3, xtol=1e-3, maxfun=1000)
        print("...finished")
        rSpcPar = plsq[2:int(len(plsq) / 2 + 2)]
        iSpcPar = plsq[int(len(plsq) / 2 + 2):]
        self.rSpc[:len(rSpcPar)] = rSpcPar
        self.iSpc[:len(iSpcPar)] = iSpcPar
        return plsq
        # end fitPhase

    #def getHist(self, spc, xaxis, plot=True, ph0=0.0, ph1=0.0):
    def getHist(self, spc, xaxis, ph0=0.0, ph1=0.0):
        hr = self.hist(spc.real)
        hi = self.hist(spc.imag)
        xr = hr[1][1:self.nbins + 1]
        xi = hi[1][1:self.nbins + 1]
        w = self.histRWeight(xr)
        hr2 = hr[0] * w
        hi2 = hi[0]
        pp = False
        if (ph0 != 0.0) or (ph1 != 0):
            pp = True

        if pp == True:
            spc2a = phase2(spc, ph0, ph1)
            hra = self.hist(spc2a.real)
            hia = self.hist(spc2a.imag)
            xra = hra[1][1:self.nbins + 1]
            xia = hia[1][1:self.nbins + 1]
            wa = self.histRWeight(xra)
            hr2a = hra[0] * wa
            hi2a = hia[0]
            maxPos = np.where(hi2a == np.max(hi2a))
            if (np.abs(maxPos[0][0] - len(hi2a) / 2) > 2):
                hi2a = hi2a * np.sign(xia)

        #if (plot == True):
        #    pl.subplot(2, 1, 1)
        #    if (pp == True):
        #        pl.plot(xra, hra[0])
        #        pl.plot(xra, hr2a)
        #
        #    pl.plot(xr, hr[0])
        #    pl.plot(xr, hr2)
        #    pl.subplot(2, 1, 2)
        #    if (pp == True):
        #        pl.plot(xia, hi2a)
        #
        #    pl.plot(xi, hi2)
        #    pl.show()

        return hr2 + 1j * hi2
        # end getHist

    def histRWeight(self, x):
        n1 = x[np.where(x < 0.0)]
        n2 = x[np.where(x < 0.05)]
        n2 = n2[np.where(n2 >= 0.0)]
        n3 = x[np.where(x < 0.5)]
        n3 = n3[np.where(n3 >= 0.05)]
        y4 = x[np.where(x >= 0.5)]
        y1 = (n1 + 0.05) * 1.5 / (1 - 0.05)
        y2 = 1 - np.abs(n2) / 0.05
        y3 = -(n3 - 0.05) * 0.5 / (0.5 - 0.05)
        y4 = -0.5 * np.ones(len(y4))
        weight = np.append(y1, y2)
        weight = np.append(weight, y3)
        weight = np.append(weight, y4)
        return weight
        # end histRWeight

    def histIWeight(self, x):
        weight = -x * x
        return weight
        # end histIWeight

    def hist(self, spc):
        h = np.histogram(spc, bins=self.nbins, range=(-1.0, 1.0))
        return h

    def phase2(elf, mat, ph0, ph1):
        npts = len(mat)
        ph0 = -ph0 * math.pi / 180.0
        ph1 = -ph1 * math.pi / 180.0
        t = complex()
        frac = np.linspace(0, 1, npts)
        ph = ph0 + frac * ph1
        mat = np.cos(ph) * mat.real + np.sin(ph) * mat.imag + 1j * (-np.sin(ph) * mat.real + np.cos(ph) * mat.imag)
        return mat
        # end phase2

    def phaseFitFunc(self, par, spc, xaxis):
        ph0 = self.mFact0 * par[0]
        ph1 = self.mFact1 * par[1]
        spc = self.phase2(spc, ph0, ph1)
        l = self.calcLine(spc, xaxis)
        spc = spc - l
        nc = int((len(par) - 2) / 2)
        cr = par[2:2 + nc]
        ci = par[2 + nc:2 + 2 * nc]
        blr = np.polynomial.chebyshev.chebval(xaxis, cr)
        bli = np.polynomial.chebyshev.chebval(xaxis, ci)
        spc = spc - blr - 1j * bli
        hr = self.hist(spc.real)
        hi = self.hist(spc.imag)
        n1 = -self.qr(hr)
        n2 = self.qi(hi)
        # print("%4.2f & %4.2f | %4.8f & %4.8f" % (n1, n2, ph0, ph1))
        return 1.0 * n1 + 100.0 * n2
        # end phaseFitFunc

    def phaseFitFuncEval(self, par, spc, xaxis): #, plot):
        ph0 = self.mFact0 * par[0]
        ph1 = self.mFact1 * par[1]
        spc = self.phase2(spc, ph0, ph1)
        l = self.calcLine(spc, xaxis)
        spc = spc - l
        nc = int((len(par) - 2) / 2)
        cr = par[2:2 + nc]
        ci = par[2 + nc:2 + 2 * nc]
        blr = np.polynomial.chebyshev.chebval(xaxis, cr)
        bli = np.polynomial.chebyshev.chebval(xaxis, ci)
        spc = spc - blr - 1j * bli
        #if (plot == True):
        #    hr = self.hist(spc.real)
        #    hi = self.hist(spc.imag)
        #    xr = hr[1][1:nbins + 1]
        #    xi = hi[1][1:nbins + 1]
        #    pl.subplot(2, 1, 1)
        #    pl.plot(xr, hr[0])
        #    pl.subplot(2, 1, 2)
        #    pl.plot(xi, hi[0])
        #    pl.show()

        return spc
        # end phaseFitFuncEval

    def qr(self, histReal):
        x = histReal[1][1:self.nbins + 1]
        w = self.histRWeight(x)
        h = histReal[0] * w
        return np.sum(h)
        # end qr

    def qi(self, histImag):
        x = histImag[1][1:self.nbins + 1]
        w = self.histIWeight(x)
        h = histImag[0] * w
        return np.sum(h)
        # end qi

    def setVars(self, spc):
        self.npts = len(spc)
        # end setVars
